@@ -49,6 +49,16 @@ main(int argc, char **argv)
 
     __pmSetProgname(argv[0]);
 
+#ifdef HAVE_GETOPT_NEEDS_POSIXLY_CORRECT
+    /*
+     * dbpmda mimics pmcd wrt POSIX getopt(2) handling, which is:
+     * "pmcd does not really need this for its own options because the
+     * arguments like "arg -x" are not valid.  But the PMDA's launched
+     * by pmcd from pmcd.conf may not be so lucky."
+     */
+    putenv("POSIXLY_CORRECT=");
+#endif
+
     iflag = isatty(0);
 
     while ((c = getopt(argc, argv, "q:D:ein:?")) != EOF) {
@@ -167,7 +177,6 @@ main(int argc, char **argv)
 			closedso();
 			break;
 		    
-		    case PDU_ASCII:
 		    case PDU_BINARY:
 			closepmda();
 			break;
@@ -185,7 +194,6 @@ main(int argc, char **argv)
 			dodso(PDU_DESC_REQ);
 			break;
 		    
-		    case PDU_ASCII:
 		    case PDU_BINARY:
 			dopmda(PDU_DESC_REQ);
 			break;
@@ -202,7 +210,6 @@ main(int argc, char **argv)
 			dodso(PDU_FETCH);
 			break;
 		    
-		    case PDU_ASCII:
 		    case PDU_BINARY:
 			dopmda(PDU_FETCH);
 			break;
@@ -219,7 +226,6 @@ main(int argc, char **argv)
 			dodso(PDU_INSTANCE_REQ);
 			break;
 		    
-		    case PDU_ASCII:
 		    case PDU_BINARY:
 			dopmda(PDU_INSTANCE_REQ);
 			break;
@@ -236,7 +242,6 @@ main(int argc, char **argv)
 			dodso(PDU_RESULT);
 			break;
 		    
-		    case PDU_ASCII:
 		    case PDU_BINARY:
 			dopmda(PDU_RESULT);
 			break;
@@ -271,7 +276,6 @@ main(int argc, char **argv)
 		    dodso(PDU_TEXT_REQ);
 		    break;
 
-		case PDU_ASCII:
 		case PDU_BINARY:
 		    dopmda(PDU_TEXT_REQ);
 		    break;
