@@ -1,5 +1,6 @@
 /*
  * Copyright (c) 1995-2002 Silicon Graphics, Inc.  All Rights Reserved.
+ * Copyright (c) 2008-2009 Aconex.  All Rights Reserved.
  * 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -10,10 +11,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  */
 
 #ifndef _IMPL_H
@@ -219,8 +216,7 @@ extern void __pmSyslog(int);
 extern void __pmNotifyErr(int, const char *, ...);
 
 /*
- * These ones are only for debugging and may not appear in the shipped
- * libpmapi ...
+ * These are for debugging only (but are present in the shipped libpcp)
  */
 EXTERN int	pmDebug;
 #define  DBG_TRACE_PDU		1	/* PDU send and receive */
@@ -252,6 +248,7 @@ extern void __pmPrintDesc(FILE *, const pmDesc *);
 extern void __pmFreeResultValues(pmResult *);
 extern const char *__pmPDUTypeStr(int);
 extern void __pmDumpNameSpace(FILE *, int);
+EXTERN int __pmLogReads;
 
 #ifdef PCP_DEBUG
 extern void __pmDumpIDList(FILE *, int, const pmID *);
@@ -525,20 +522,6 @@ typedef struct {
 
 #define __PM_MODE_MASK	0xffff
 
-#define pmXTBdeltaToTimeval(d, m, t) { \
-    (t)->tv_sec = 0; \
-    (t)->tv_usec = (long)0; \
-    switch(PM_XTB_GET(m)) { \
-    case PM_TIME_NSEC: (t)->tv_usec = (long)((d) / 1000); break; \
-    case PM_TIME_USEC: (t)->tv_usec = (long)(d); break; \
-    case PM_TIME_MSEC: (t)->tv_sec = (d) / 1000; (t)->tv_usec = (long)(1000 * ((d) % 1000)); break; \
-    case PM_TIME_SEC: (t)->tv_sec = (d); break; \
-    case PM_TIME_MIN: (t)->tv_sec = (d) * 60; break; \
-    case PM_TIME_HOUR: (t)->tv_sec = (d) * 360; break; \
-    default: (t)->tv_sec = (d) / 1000; (t)->tv_usec = (long)(1000 * ((d) % 1000)); break; \
-    } \
-}
-
 #define PM_CONTEXT_FREE		-1	/* special type */
 
 /* handle to __pmContext pointer */
@@ -577,12 +560,7 @@ typedef struct {
 #define CVERSION        1
 #define CAUTH           2
 
-/* ASCII PDU line buffer */
-#define ABUFSIZE 256
-extern char __pmAbuf[ABUFSIZE];
-
 extern int __pmXmitPDU(int, __pmPDU *);
-extern int __pmXmitAscii(int, const char *, int);
 extern int __pmGetPDU(int, int, int, __pmPDU **);
 extern int __pmGetPDUCeiling (void);
 extern int __pmSetPDUCeiling (int);
@@ -595,9 +573,7 @@ extern void __pmSetPDUCntBuf(unsigned *, unsigned *);
 #define TIMEOUT_NEVER	 0
 #define TIMEOUT_DEFAULT	-1
 #define GETPDU_ASYNC	-2
-extern int __pmRecvPDU(int, int, __pmPDU **);	/* old interface to __pmGetPDU */
 
-extern int __pmRecvLine(__pmPDU *, int, char *);
 extern __pmPDU *__pmFindPDUBuf(int);
 extern void __pmPinPDUBuf(void *);
 extern int __pmUnpinPDUBuf(void *);
@@ -624,6 +600,11 @@ extern void __pmCountPDUBuf(int, int *, int *);
 #define PDU_FINISH		0x7010
 #define PDU_MAX		 	(PDU_FINISH - PDU_START)
 
+/*
+ * Unit of space allocation for PDU buffer.
+ */
+#define PDU_CHUNK		1024
+
 /*
  * PDU encoding formats
  */
@@ -815,18 +796,6 @@ extern int __pmLogChangeVol(__pmLogCtl *, int);
 extern int __pmLogChkLabel(__pmLogCtl *, FILE *, __pmLogLabel *, int);
 extern int __pmGetArchiveEnd(__pmLogCtl *, struct timeval *);
 
-#if !defined(PM_LOG_PORT_DIR)
-/* This is the default directory where pmlogger creates the file containing the port
- * number it is using. It may be overridden in platform_defs.h
- */
-#define	PM_LOG_PORT_DIR		"/var/tmp/pmlogger"
-#endif
-
-/* The primary logger creates a symbolic link to its own port file
- * in PM_LOG_PORT_DIR.  This is the name of the link.
- */
-#define PM_LOG_PRIMARY_LINK	"primary"
-
 /* struct for maintaining information about pmlogger ports */
 typedef struct {
     int		pid;		/* process id of logger */
@@ -876,14 +845,9 @@ extern double __pmtimevalAdd(const struct timeval *, const struct timeval *);
 extern double __pmtimevalSub(const struct timeval *, const struct timeval *);
 extern double __pmtimevalToReal(const struct timeval *);
 extern void __pmtimevalFromReal(double, struct timeval *);
+extern void __pmtimevalSleep(struct timeval);
+extern void __pmtimevalPause(struct timeval);
 
-/*
- * interface to __pmGetPDU() controls
- */
-extern int __pmMoreInput(int);
-extern void __pmNoMoreInput(int);
-
-/* time control timezone */
 typedef struct {
     char		*label;		/* label to name tz */
     char		*tz;		/* env $TZ */
@@ -900,7 +864,6 @@ extern void __pmEventTrace(const char *);
  */
 
 typedef int (*__pmConnectHostType)(int, int);
-extern __pmConnectHostType __pmConnectHostMethod;
 
 extern int __pmSetSocketIPC(int);
 extern int __pmSetVersionIPC(int, int);
@@ -1078,6 +1041,28 @@ extern int __pmProcessCreate(char **, int *, int *);
 extern int __pmProcessDataSize(unsigned long *);
 extern int __pmProcessRunTimes(double *, double *);
 
+/*
+ * platform independent memory mapped file handling
+ */
+extern void *__pmMemoryMap(int, size_t, int);
+extern void __pmMemoryUnmap(void *, size_t);
+
+/*
+ * platform independent signal handling
+ */
+typedef void (*__pmSignalHandler)(int);
+extern int __pmSetSignalHandler(int, __pmSignalHandler);
+
+/*
+ * platform independent environment and filesystem path access
+ */
+typedef void (*__pmConfigCallback)(char *, char *, char *);
+EXTERN __pmConfigCallback __pmNativeConfig;
+extern void __pmConfig(const char *, __pmConfigCallback);
+extern char *__pmNativePath(char *);
+extern int __pmAbsolutePath(char *);
+extern int __pmPathSeparator();
+
 /*
  * AF - general purpose asynchronous event management routines
  */
@@ -1172,6 +1157,13 @@ extern int __pmControlLog(int, const pmResult *, int, int, int, pmResult **);
 #define PMLC_GET_DELTA(val) \
         (((val & ~0xf) >> 4) & PMLC_MAX_DELTA)
 
+
+/*
+ * helper function to register client identity with pmcd for export
+ * via pmcd.client.whoami
+ */
+extern int __pmSetClientId(char *);
+
 #ifdef __cplusplus
 }
 #endif
