@@ -14,20 +14,12 @@
  * You should have received a copy of the GNU Lesser General Public License
  * along with this library; if not, write to the Free Software Foundation,
  * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.
- * 
- * Contact information: Silicon Graphics, Inc., 1500 Crittenden Lane,
- * Mountain View, CA 94043, USA, or: http://www.sgi.com
  */
 
-#ident "$Id: instance.c,v 1.3 2006/05/02 05:57:28 makc Exp $"
-
-#include <stdio.h>
 #include "pmapi.h"
 #include "impl.h"
 #include "pmda.h"
 
-extern int	errno;
-
 static int
 request_instance (__pmContext *ctxp, pmInDom indom, int inst, const char *name)
 {
