@@ -20,11 +20,11 @@
 #include "pmapi.h"
 #include "impl.h"
 
-int		__pmLogReads = 0;
+INTERN int	__pmLogReads = 0;
 
-static char	*logfilename = NULL;
-static int	logfilenamelen = 0;
-static int	seeking_end = 0;
+static char	*logfilename;
+static int	logfilenamelen;
+static int	seeking_end;
 
 /*
  * first two fields are made to look like a pmValueSet when no values are
@@ -579,6 +579,7 @@ __pmLogLoadLabel(__pmLogCtl *lcp, const char *name)
     int		sts;
     int		blen;
     int		exists = 0;
+    int		sep = __pmPathSeparator();
     char	*q;
     char	*base;
     char	*tbuf;
@@ -622,7 +623,8 @@ __pmLogLoadLabel(__pmLogCtl *lcp, const char *name)
 	    else if (strcmp(q, ".meta") == 0) strip = 1;
 	    else if (q[1] != '\0') {
 		char	*end;
-		(void)strtol(q+1, &end, 10);
+		long	tmpl;	/* NOTUSED, pander to gcc */
+		tmpl = strtol(q+1, &end, 10);
 		if (*end == '\0') strip = 1;
 	    }
 	}
@@ -631,7 +633,7 @@ __pmLogLoadLabel(__pmLogCtl *lcp, const char *name)
 	}
     }
 
-    snprintf(filename, sizeof(filename), "%s/%s", dir, base);
+    snprintf(filename, sizeof(filename), "%s%c%s", dir, sep, base);
     if ((lcp->l_name = strdup(filename)) == NULL) {
 	sts = -oserror();
 	free(tbuf);
@@ -661,14 +663,14 @@ __pmLogLoadLabel(__pmLogCtl *lcp, const char *name)
 		continue;
 #ifdef PCP_DEBUG
 	    if (pmDebug & DBG_TRACE_LOG) {
-		snprintf(filename, sizeof(filename), "%s/%s", dir, direntp->d_name);
+		snprintf(filename, sizeof(filename), "%s%c%s", dir, sep, direntp->d_name);
 		fprintf(stderr, "__pmLogOpen: inspect file \"%s\"\n", filename);
 	    }
 #endif
 	    tp = &direntp->d_name[blen+1];
 	    if (strcmp(tp, "index") == 0) {
 		exists = 1;
-		snprintf(filename, sizeof(filename), "%s/%s", dir, direntp->d_name);
+		snprintf(filename, sizeof(filename), "%s%c%s", dir, sep, direntp->d_name);
 		if ((lcp->l_tifp = fopen(filename, "r")) == NULL) {
 		    sts = -errno;
 		    goto cleanup;
@@ -676,7 +678,7 @@ __pmLogLoadLabel(__pmLogCtl *lcp, const char *name)
 	    }
 	    else if (strcmp(tp, "meta") == 0) {
 		exists = 1;
-		snprintf(filename, sizeof(filename), "%s/%s", dir, direntp->d_name);
+		snprintf(filename, sizeof(filename), "%s%c%s", dir, sep, direntp->d_name);
 		if ((lcp->l_mdfp = fopen(filename, "r")) == NULL) {
 		    sts = -errno;
 		    goto cleanup;
@@ -1901,7 +1903,7 @@ __pmGetArchiveEnd(__pmLogCtl *lcp, struct timeval *tp)
 {
     struct stat	sbuf;
     FILE	*f;
-    long	save;
+    long	save = 0;
     pmResult	*rp = NULL;
     pmResult	*nrp;
     int		i;
@@ -1911,7 +1913,7 @@ __pmGetArchiveEnd(__pmLogCtl *lcp, struct timeval *tp)
     long	offset;
     int		vol;
     __pm_off_t	logend;
-    __pm_off_t	physend;
+    __pm_off_t	physend = 0;
 
     /*
      * expect things to be stable, so l_maxvol is not empty, and
