@@ -33,36 +33,23 @@ __pmSendTextReq(int fd, int mode, int ident, int type)
 {
     text_req_t	*pp;
 
-    if (mode == PDU_BINARY) {
+    if (mode == PDU_ASCII)
+	return PM_ERR_NOASCII;
+    if ((pp = (text_req_t *)__pmFindPDUBuf(sizeof(text_req_t))) == NULL)
+	return -errno;
+    pp->hdr.len = sizeof(text_req_t);
+    pp->hdr.type = PDU_TEXT_REQ;
 
-	if ((pp = (text_req_t *)__pmFindPDUBuf(sizeof(text_req_t))) == NULL)
-	    return -errno;
-	pp->hdr.len = sizeof(text_req_t);
-	pp->hdr.type = PDU_TEXT_REQ;
+    if (type & PM_TEXT_PMID)
+	pp->ident = __htonpmID((pmID)ident);
+    else if (type & PM_TEXT_INDOM)
+	pp->ident = __htonpmInDom((pmInDom)ident);
+    else
+	pp->ident = __htonpmInDom((pmInDom)ident);
 
-	if (type & PM_TEXT_PMID) {
-	    pp->ident = __htonpmID((pmID)ident);
-	}
-	else
-	if (type & PM_TEXT_INDOM) {
-	    pp->ident = __htonpmInDom((pmInDom)ident);
-	}
+    pp->type = htonl(type);
 
-	pp->type = htonl(type);
-
-	return __pmXmitPDU(fd, (__pmPDU *)pp);
-    }
-    else {
-	/* assume PDU_ASCII */
-	int		nbytes, sts;
-
-	snprintf(__pmAbuf, sizeof(__pmAbuf), "TEXT_REQ %d %d\n", ident, type);
-	nbytes = (int)strlen(__pmAbuf);
-	sts = __pmXmitAscii(fd, __pmAbuf, nbytes);
-	if (sts < 0)
-	    return sts;
-	return 0;
-    }
+    return __pmXmitPDU(fd, (__pmPDU *)pp);
 }
 
 int
@@ -70,10 +57,8 @@ __pmDecodeTextReq(__pmPDU *pdubuf, int mode, int *ident, int *type)
 {
     text_req_t	*pp;
 
-    if (mode == PDU_ASCII) {
-	/* Incoming ASCII request PDUs not supported */
+    if (mode == PDU_ASCII)
 	return PM_ERR_NOASCII;
-    }
 
     pp = (text_req_t *)pdubuf;
     *type = ntohl(pp->type);
@@ -103,10 +88,8 @@ __pmSendText(int fd, int mode, int ident, const char *buffer)
     text_t	*pp;
     size_t	need;
 
-    if (mode == PDU_ASCII) {
-	/* Outgoing ASCII result PDUs not supported */
+    if (mode == PDU_ASCII)
 	return PM_ERR_NOASCII;
-    }
 
     need = sizeof(text_t) - sizeof(pp->buffer) + sizeof(__pmPDU)*((strlen(buffer) - 1 + sizeof(__pmPDU))/sizeof(__pmPDU));
     if ((pp = (text_t *)__pmFindPDUBuf((int)need)) == NULL)
@@ -133,67 +116,24 @@ int
 __pmDecodeText(__pmPDU *pdubuf, int mode, int *ident, char **buffer)
 {
     text_t	*pp;
+    int		buflen;
 
-    if (mode == PDU_BINARY) {
-	int		buflen;
-	pp = (text_t *)pdubuf;
-	/*
-	 * Note: ident argument is returned in network byte order.
-	 * The caller has to convert it to host byte order because
-	 * the type of ident is not part of the transmitted PDU_TEXT
-	 * pdu (ident may be either a pmID or a pmInDom. The caller
-	 * must use either __ntohpmID() or __ntohpmInDom(), respectfully.
-	 */
-	*ident = pp->ident;
-
-	buflen = ntohl(pp->buflen);
-	if ((*buffer = (char *)malloc(buflen+1)) == NULL)
-	    return -errno;
-	strncpy(*buffer, pp->buffer, buflen);
-	(*buffer)[buflen] = '\0';
-    }
-    else {
-	/* assume PDU_ASCII */
-	int		n;
-	int		len;
-	int		type;
-	int		nbyte;
-	char		*p;
-
-	n = __pmRecvLine(pdubuf, ABUFSIZE, __pmAbuf);
-	if (n <= 0)
-	    return n;
-	if ((n = sscanf(__pmAbuf, "TEXT %d %d %d", ident, &type, &nbyte)) != 3) {
-	    __pmNotifyErr(LOG_WARNING, "__pmDecodeText: ASCII botch %d values from: \"%s\"\n", n, __pmAbuf);
-	    return PM_ERR_IPC;
-	}
-	if ((*buffer = (char *)malloc(nbyte+1)) == NULL)
-	    return -errno;
-	p = *buffer;
-	len = nbyte;
-	while (len) {
-	    n = __pmRecvLine(pdubuf, ABUFSIZE, __pmAbuf);
-	    if (n <= 0) {
-		free(*buffer);
-		return n;
-	    }
-	    /* -1 here as follows: -2 for leading ". " +1 for stripped \n */
-	    n = (int)strlen(__pmAbuf) - 1;
-	    len -= n;
-	    if (len < 0) {
-		__pmNotifyErr(LOG_WARNING, "__pmDecodeText: ASCII botch, more than %d bytes of text, ...\nat or after: \"%s\"\n", nbyte, __pmAbuf);
-		free(*buffer);
-		return PM_ERR_IPC;
-	    }
-	    strcpy(p, &__pmAbuf[2]);
-	    p += n;
-	    p[-1] = '\n';	/* put back what was stripped by __pmRecvLine */
-	}
-	if (p > *buffer && (type & PM_TEXT_ONELINE))
-	    /* no trailing \n for non-null one-line fellows */
-	    p--;
-	*p = '\0';
-    }
+    if (mode == PDU_ASCII)
+	return PM_ERR_NOASCII;
 
+    pp = (text_t *)pdubuf;
+    /*
+     * Note: ident argument is returned in network byte order.
+     * The caller has to convert it to host byte order because
+     * the type of ident is not part of the transmitted PDU_TEXT
+     * pdu (ident may be either a pmID or a pmInDom. The caller
+     * must use either __ntohpmID() or __ntohpmInDom(), respectfully.
+     */
+    *ident = pp->ident;
+    buflen = ntohl(pp->buflen);
+    if ((*buffer = (char *)malloc(buflen+1)) == NULL)
+	return -errno;
+    strncpy(*buffer, pp->buffer, buflen);
+    (*buffer)[buflen] = '\0';
     return 0;
 }
