@@ -2,6 +2,7 @@
  * General Utility Routines
  *
  * Copyright (c) 1995-2002,2004 Silicon Graphics, Inc.  All Rights Reserved.
+ * Copyright (c) 2009 Aconex.  All Rights Reserved.
  * 
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -12,10 +13,6 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  * License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.
  */
 
 #include <stdarg.h>
@@ -31,48 +28,23 @@
 #if defined(HAVE_SYS_TIMES_H)
 #include <sys/times.h>
 #endif
+#if defined(HAVE_SYS_MMAN_H)
+#include <sys/mman.h> 
+#endif
 #if defined(HAVE_IEEEFP_H)
 #include <ieeefp.h>
 #endif
 
-static FILE	**filelog = NULL;
-static int	nfilelog = 0;
-static int	dosyslog = 0;
+static FILE	**filelog;
+static int	nfilelog;
+static int	dosyslog;
 static int	pmState = PM_STATE_APPL;
-static int	done_exit = 0;
+static int	done_exit;
 
 INTERN char	*pmProgname = "pcp";		/* the real McCoy */
 
 static int vpmprintf(const char *, va_list);
 
-int
-__pmSetProgname(const char *program)
-{
-    char	*p;
-
-    /* Trim command name of leading directory components */
-    if (program)
-	pmProgname = (char *)program;
-    for (p = pmProgname; pmProgname && *p; p++) {
-	if (*p == '/')
-	    pmProgname = p+1;
-    }
-#if defined(IS_MINGW)
-    _fmode = O_BINARY;
-
-    int sts;
-    WORD wVersionRequested = MAKEWORD(2, 2);
-    WSADATA wsaData;
-
-    /* If Windows networking is not setup, all networking calls fail;
-     * this even includes gethostname(2), if you can believe that. :[
-     */
-    if ((sts = WSAStartup(wVersionRequested, &wsaData)) != 0)
-	return sts;
-#endif
-    return 0;
-}
-
 /*
  * if onoff == 1, logging is to syslog and stderr, else logging is
  * just to stderr (this is the default)
@@ -146,6 +118,7 @@ __pmNotifyErr(int priority, const char *message, ...)
 
     pmprintf("[%.19s] %s(%d) %s: ", ctime(&now), pmProgname, getpid(), level);
     vpmprintf(message, arg);
+    va_end(arg);
     /* trailing \n if needed */
     for (p = (char *)message; *p; p++)
 	;
@@ -584,7 +557,7 @@ __pmNoMem(const char *where, size_t size, int fatal)
 }
 
 /*
- * this one is used just below the PMAPI to convert some Unix errors
+ * this one is used just below the PMAPI to convert platform errors
  * into more appropriate PMAPI error codes
  */
 int
@@ -592,6 +565,10 @@ __pmMapErrno(int sts)
 {
     if (sts == -EBADF || sts == -EPIPE)
 	sts = PM_ERR_IPC;
+#ifdef IS_MINGW
+    else if (sts == -EINVAL)
+	sts = PM_ERR_IPC;
+#endif
     return sts;
 }
 
@@ -828,7 +805,7 @@ pmfstate(int state)
 	if ((ferr = getenv("PCP_STDERR")) != NULL) {
 	    if (strcasecmp(ferr, "DISPLAY") == 0) {
 		char * xconfirm = pmGetConfig("PCP_XCONFIRM_PROG");
-		if (access(xconfirm, X_OK) < 0) {
+		if (access(__pmNativePath(xconfirm), X_OK) < 0) {
 		    fprintf(stderr, "%s: using stderr - cannot access %s: %s\n",
 			    pmProgname, xconfirm, strerror(errno));
 		}
@@ -850,7 +827,8 @@ vpmprintf(const char *msg, va_list arg)
     if (fptr == NULL && msgsize == 0) {		/* create scratch file */
 	int	fd = -1;
 
-	if ((fname = tmpnam(NULL)) == NULL ||
+	fname = tempnam(pmGetConfig("PCP_TMP_DIR"), "pcp-");
+	if (fname == NULL ||
 	    (fd = open(fname, O_RDWR|O_APPEND|O_CREAT|O_EXCL, 0600)) < 0 ||
 	    (fptr = fdopen(fd, "a")) == NULL) {
 	    fprintf(stderr, "%s: vpmprintf: failed to create \"%s\": %s\n",
@@ -889,8 +867,9 @@ int
 pmflush(void)
 {
     int		sts = 0;
+    int		len;
     int		state;
-    FILE	*eptr;
+    FILE	*eptr = NULL;
 
     if (fptr != NULL && msgsize > 0) {
 	fflush(fptr);
@@ -905,14 +884,20 @@ pmflush(void)
 	switch (state) {
 	case PM_USESTDERR:
 	    rewind(fptr);
-	    while ((sts = (int)read(fileno(fptr), outbuf, MSGBUFLEN)) > 0)
-		write(fileno(stderr), outbuf, sts);
+	    while ((len = (int)read(fileno(fptr), outbuf, MSGBUFLEN)) > 0) {
+		sts = write(fileno(stderr), outbuf, len);
+		if (sts != len) {
+		    fprintf(stderr, "pmflush: write() failed: %s\n", 
+			strerror(errno));
+		}
+		sts = 0;
+	    }
 	    break;
 	case PM_USEDIALOG:
 	    /* If we're here, it means xconfirm has passed access test */
 	    snprintf(outbuf, sizeof(outbuf), "%s -file %s -c -B OK -icon info"
 		    " %s -header 'PCP Information' >/dev/null",
-		    pmGetConfig("PCP_XCONFIRM_PROG"), fname,
+		    __pmNativePath(pmGetConfig("PCP_XCONFIRM_PROG")), fname,
 		    (msgsize > 80 ? "-useslider" : ""));
 	    if (system(outbuf) < 0) {
 		fprintf(stderr, "%s: system failed: %s\n", pmProgname,
@@ -922,14 +907,21 @@ pmflush(void)
 	    break;
 	case PM_USEFILE:
 	    rewind(fptr);
-	    while ((sts = (int)read(fileno(fptr), outbuf, MSGBUFLEN)) > 0)
-		write(fileno(eptr), outbuf, sts);
+	    while ((len = (int)read(fileno(fptr), outbuf, MSGBUFLEN)) > 0) {
+		sts = write(fileno(eptr), outbuf, len);
+		if (sts != len) {
+		    fprintf(stderr, "pmflush: write() failed: %s\n", 
+			strerror(errno));
+		}
+		sts = 0;
+	    }
 	    fclose(eptr);
 	    break;
 	}
 	fclose(fptr);
 	fptr = NULL;
 	unlink(fname);
+	free(fname);
 	if (sts >= 0)
 	    sts = msgsize;
     }
@@ -939,6 +931,76 @@ pmflush(void)
     return sts;
 }
 
+/*
+ * Set the pmcd client identity as exported by pmcd.client.whoami
+ *
+ * Itentity is of the form
+ *	hostname (ipaddr) <id>
+ *
+ * Assumes you already have a current host context.
+ */
+int
+__pmSetClientId(char *id)
+{
+    char		*name = "pmcd.client.whoami";
+    pmID		pmid;
+    int			sts;
+    pmResult		res;
+    pmValueSet		pmvs;
+    pmValueBlock	*pmvb;
+    char        	host[MAXHOSTNAMELEN];
+    char        	ipaddr[16] = "";	/* IPv4 xxx.xxx.xxx.xxx */
+    struct hostent      *hep = NULL;
+    int			vblen;
+
+    if ((sts = pmLookupName(1, &name, &pmid)) < 0)
+	return sts;
+
+    (void)gethostname(host, MAXHOSTNAMELEN);
+    hep = gethostbyname(host);
+    if (hep != NULL) {
+        strcpy(host, hep->h_name);
+	if (hep->h_addrtype == AF_INET) {
+	    strcpy(ipaddr, inet_ntoa(*((struct in_addr *)hep->h_addr_list[0])));
+	}
+	vblen = strlen(host) + strlen(ipaddr) + strlen(id) + 5;
+    }
+    else
+	vblen = strlen(host) + strlen(id) + 2;
+
+    /* build pmResult for pmStore() */
+    pmvb = (pmValueBlock *)malloc(PM_VAL_HDR_SIZE+vblen);
+    if (pmvb == NULL) {
+	__pmNoMem("__pmSetClientId", PM_VAL_HDR_SIZE+vblen, PM_RECOV_ERR);
+	return -ENOMEM;
+    }
+    pmvb->vtype = PM_TYPE_STRING;
+    pmvb->vlen = PM_VAL_HDR_SIZE+vblen;
+    strcpy(pmvb->vbuf, host);
+    strcat(pmvb->vbuf, " ");
+    if (ipaddr[0] != '\0') {
+	strcat(pmvb->vbuf, "(");
+	strcat(pmvb->vbuf, ipaddr);
+	strcat(pmvb->vbuf, ") ");
+    }
+    strcat(pmvb->vbuf, id);
+
+    pmvs.pmid = pmid;
+    pmvs.numval = 1;
+    pmvs.valfmt = PM_VAL_SPTR;
+    pmvs.vlist[0].value.pval = pmvb;
+    pmvs.vlist[0].inst = PM_IN_NULL;
+
+    res.numpmid = 1;
+    res.vset[0] = &pmvs;
+
+    sts = pmStore(&res);
+    free(pmvb);
+
+    return 0;
+}
+
+
 /*
  * Support for C environments that have lame libc implementations.
  * All of these developed from first principles, so no 3rd party
@@ -989,26 +1051,6 @@ isnand(double d)
 }
 #endif
 
-#ifndef HAVE_UNSETENV
-int
-unsetenv(const char *name)
-{
-    extern char **_environ;
-    char	**ep;
-    int		len = (int)strlen(name);
-    int		found = 0;
-
-    for (ep = _environ; *ep != NULL; ep++) {
-	if (strncmp(*ep, name, len) == 0 && (*ep)[len] == '=') {
-	    found = 1;
-	}
-	if (found)
-	    ep[0] = ep[1];
-    }
-    return found;
-}
-#endif
-
 #ifndef HAVE_SCANDIR
 /*
  * Scan the directory dirname, building an array of pointers to
@@ -1156,8 +1198,10 @@ __pmProcessCreate(char **argv, int *infd, int *outfd)
     int		out[2];
     pid_t	pid;
 
-    pipe(in);
-    pipe(out);
+    if (pipe1(in) < 0)
+	return -errno;
+    if (pipe1(out) < 0)
+	return -errno;
 
     pid = fork();
     if (pid < 0) {
@@ -1190,4 +1234,42 @@ __pmProcessCreate(char **argv, int *infd, int *outfd)
     }
     return 0;
 }
+
+int
+__pmSetSignalHandler(int sig, __pmSignalHandler func)
+{
+    signal(sig, func);
+    return 0;
+}
+
+int
+__pmSetProgname(const char *program)
+{
+    char	*p;
+
+    /* Trim command name of leading directory components */
+    if (program)
+	pmProgname = (char *)program;
+    for (p = pmProgname; pmProgname && *p; p++) {
+	if (*p == '/')
+	    pmProgname = p+1;
+    }
+    return 0;
+}
+
+void *
+__pmMemoryMap(int fd, size_t sz, int writable)
+{
+    int mflags = writable ? (PROT_READ | PROT_WRITE) : PROT_READ;
+    void *addr = mmap(NULL, sz, mflags, MAP_SHARED, fd, 0);
+    if (addr == MAP_FAILED)
+	return NULL;
+    return addr;
+}
+
+void
+__pmMemoryUnmap(void *addr, size_t sz)
+{
+    munmap(addr, sz);
+}
 #endif
