@@ -41,10 +41,6 @@ typedef struct {
 typedef struct {
     int		dir_fd;
     int		pag_fd;
-#if defined(IS_MINGW)
-    HANDLE	handle;
-    HANDLE	texthandle;
-#endif
     int		numidx;
     help_idx_t	*index;
     char	*text;
