@@ -14,13 +14,8 @@
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
- * 
- * Contact information: Silicon Graphics, Inc., 1500 Crittenden Lane,
- * Mountain View, CA 94043, USA, or: http://www.sgi.com
  */
 
-#ident "$Id: newhelp.c,v 1.6 2003/02/20 19:20:25 kenmcd Exp $"
-
 /*
  * newhelp file
  *
@@ -46,7 +41,6 @@
 #define MAXLINE	128
 #define MAXENTRY 1024
 
-extern int	errno;
 static int	verbose = 0;
 static int	ln = 0;
 static char	*filename;
@@ -143,7 +137,6 @@ newentry(char *buf)
 	    numindex *= 2;
 	if ((hindex = (help_idx_t *)realloc(hindex, numindex * sizeof(hindex[0]))) == NULL) {
 	    __pmNoMem("newentry", numindex * sizeof(hindex[0]), PM_FATAL_ERR);
-	    /*NOTREACHED*/
 	}
     }
 
@@ -233,9 +226,7 @@ idcomp(const void *a, const void *b)
 }
 
 int
-main(argc, argv)
-int argc;
-char *argv[];
+main(int argc, char **argv)
 {
     int		n;
     int		c;
@@ -252,16 +243,8 @@ char *argv[];
     char	*p;
     int		skip;
     help_idx_t	hdr;
-    extern char	*optarg;
-    extern int	optind;
-    extern int	pmDebug;
 
-    /* trim command name of leading directory components */
-    pmProgname = argv[0];
-    for (p = pmProgname; *p; p++) {
-	if (*p == '/')
-	    pmProgname = p+1;
-    }
+    __pmSetProgname(argv[0]);
 
     while ((c = getopt(argc, argv, "D:n:o:Vv:?")) != EOF) {
 	switch (c) {
@@ -469,18 +452,4 @@ Options:\n\
     }
 
     exit(status);
-    /*NOTREACHED*/
 }
-
-#ifdef BUILDTOOL
-/*
- * redefine __pmGetLicenseCap(), to avoid PCP license hassles in the
- * build environment
- */
-
-int
-__pmGetLicenseCap(void)
-{
-    return PM_LIC_PCP | PM_LIC_COL | PM_LIC_MON;
-}
-#endif
