@@ -70,7 +70,7 @@ rm -f $tmp
 
 # chown/chmod is not always helpful
 #
-if [ "$PCP_PLATFORM" = cygwin -o "$PCP_PLATFORM" = mingw ]
+if [ "$PCP_PLATFORM" = mingw ]
 then
     CHOWN=": skip chown"
     CHMOD=chmod
@@ -124,7 +124,7 @@ pipe_opt=true
 #	If daemon, socket?  and default for Internet sockets?
 socket_opt=false
 socket_inet_def=''
-#	IPC Protocol for daemon (binary or text)
+#	IPC Protocol for daemon (binary only now)
 ipc_prot=binary
 #	Delay after install before checking (sec)
 check_delay=3
@@ -307,7 +307,7 @@ END					{ exit status }'
 	#
 	if pminfo -h localhost -v pmcd.version >/dev/null 2>&1
 	then
-	    $PCP_KILLALL_PROG -HUP pmcd >/dev/null 2>&1
+	    pmsignal -a -s HUP pmcd >/dev/null 2>&1
 	    __wait_for_pmcd
 	    $__pmcd_is_dead && __restore_pmcd
   	    # give PMDA a chance to cleanup, especially if a new one about
@@ -317,20 +317,19 @@ END					{ exit status }'
     fi
     rm -f /tmp/$$.pmcd.conf
 
-    # kill off any matching PMDA that is still running
+    # stop any matching PMDA that is still running
     #
-    for __sig in INT TERM KILL
+    for __sig in TERM KILL
     do
 	__pids=`_get_pids_by_name pmda$1`
 	if [ ! -z "$__pids" ]
 	then
-	    kill -$__sig $__pids >/dev/null 2>&1
+	    pmsignal -s $__sig $__pids >/dev/null 2>&1
 	    sleep 3
 	else
 	    break
 	fi
     done
-
 }
 
 # __pmda_add "entry for $PCP_PMCDCONF_PATH"
@@ -388,7 +387,7 @@ $1=="'$myname'" && $2=="'$mydomain'"	{ next }
     #
     if pminfo -h localhost -v pmcd.version >/dev/null 2>&1
     then
-	$PCP_KILLALL_PROG -HUP pmcd >/dev/null 2>&1
+	pmsignal -a -s HUP pmcd >/dev/null 2>&1
 	__wait_for_pmcd
 	$__pmcd_is_dead && __restore_pmcd
     else
@@ -692,7 +691,7 @@ __choose_ipc()
 
     if [ $ipc_type = pipe ]
     then
-	type="pipe	$ipc_prot 		$_dir/$pmda_name"
+	type="pipe	binary 		$_dir/$pmda_name"
     else
 	while true
 	do
@@ -754,6 +753,16 @@ END		{ if (warn) printf "%d warnings, ",warn
 		}'
 }
 
+__strip_pcp_dir()
+{
+    if [ "X$PCP_DIR" = X ]
+    then
+	echo "$1"
+    else
+	echo "$1" | sed -e "s,^$PCP_DIR,,"
+    fi
+}
+
 _setup()
 {
     # some more configuration controls
@@ -761,10 +770,11 @@ _setup()
     pmda_name=pmda$iam
     dso_suffix=so
     [ "$PCP_PLATFORM" = darwin ] && dso_suffix=dylib
-    [ "$PCP_PLATFORM" = cygwin -o "$PCP_PLATFORM" = mingw ] && dso_suffix=dll
+    [ "$PCP_PLATFORM" = mingw  ] && dso_suffix=dll
     dso_name="${PCP_PMDAS_DIR}/${iam}/pmda_${iam}.${dso_suffix}"
+    dso_name=`__strip_pcp_dir "$dso_name"`
     dso_entry=${iam}_init
-    pmda_dir="${PCP_PMDAS_DIR}/${iam}"
+    pmda_dir=`__strip_pcp_dir "${PCP_PMDAS_DIR}/${iam}"`
 
     # check the user is root 
     #
@@ -775,6 +785,7 @@ _setup()
     if $perl_opt
     then
 	perl_name="${PCP_PMDAS_DIR}/${iam}/pmda${iam}.pl"
+	perl_name=`__strip_pcp_dir "$perl_name"`
 	perl_pmns="${PCP_PMDAS_DIR}/${iam}/pmns"
 	perl_dom="${PCP_PMDAS_DIR}/${iam}/domain.h"
 	test -s "$perl_dom" || \
@@ -799,6 +810,30 @@ _setup()
     esac
 }
 
+_install_views()
+{
+    viewer="$1"
+    have_views=false
+
+    [ `echo *.$viewer` != "*.$viewer" ] && have_views=true
+    if [ -d $PCP_VAR_DIR/config/$viewer ]
+    then
+	$have_views && echo "Installing $viewer view(s) ..."
+	for __i in *.$viewer
+	do
+	    if [ "$__i" != "*.$viewer" ]
+	    then
+		__dest=$PCP_VAR_DIR/config/$viewer/`basename $__i .$viewer`
+		rm -f $__dest
+		cp $__i $__dest
+	    fi
+	done
+    else
+	$have_views && \
+	echo "Skip installing $viewer view(s) ... no \"$PCP_VAR_DIR/config/$viewer\" directory"
+    fi
+}
+
 # Configurable PMDA installation
 #
 # before calling _install,
@@ -854,7 +889,7 @@ _install()
             cd $PMNSDIR
 	    if pmnsdel -n $PMNSROOT $__n >$tmp 2>&1
 	    then
-		$PCP_KILLALL_PROG -HUP pmcd >/dev/null 2>&1
+		pmsignal -a -s HUP pmcd >/dev/null 2>&1
 		# Make sure the PMNS timestamp will be different the next
 		# time the PMNS is updated (for Linux only 1 sec resolution)
 		sleep 2
@@ -894,7 +929,7 @@ _install()
         cd $PMNSDIR
 	if pmnsadd -n $PMNSROOT $__n
 	then
-	    $PCP_KILLALL_PROG -HUP pmcd >/dev/null 2>&1
+	    pmsignal -a -s HUP pmcd >/dev/null 2>&1
 	    # Make sure the PMNS timestamp will be different the next
 	    # time the PMNS is updated (for Linux only 1 sec resolution)
 	    sleep 2
@@ -910,43 +945,9 @@ _install()
     trap "rm -f $tmp $tmp.*; exit" 0 1 2 3 15
     $PCP_SHARE_DIR/lib/unlockpmns $NAMESPACE
 
-    if [ -d $PCP_VAR_DIR/config/pmchart ]
-    then
-	if [ `echo *.pmchart` != "*.pmchart" ]
-	then
-	    echo "Installing pmchart view(s) ..."
-	fi
-	for __i in *.pmchart
-	do
-	    if [ "$__i" != "*.pmchart" ]
-	    then
-		__dest=$PCP_VAR_DIR/config/pmchart/`basename $__i .pmchart`
-		rm -f $__dest
-		cp $__i $__dest
-	    fi
-	done
-    else
-	echo "Skip installing pmchart view(s) ... no \"$PCP_VAR_DIR/config/pmchart\" directory"
-    fi
-
-    if [ -d $PCP_VAR_DIR/config/kmchart ]
-    then
-	if [ `echo *.kmchart` != "*.kmchart" ]
-	then
-	    echo "Installing kmchart view(s) ..."
-	fi
-	for __i in *.kmchart
-	do
-	    if [ "$__i" != "*.kmchart" ]
-	    then
-		__dest=$PCP_VAR_DIR/config/kmchart/`basename $__i .kmchart`
-		rm -f $__dest
-		cp $__i $__dest
-	    fi
-	done
-    else
-	echo "Skip installing kmchart view(s) ... no \"$PCP_VAR_DIR/config/kmchart\" directory"
-    fi
+    _install_views pmchart
+    _install_views kmchart
+    _install_views pmview
 
     if $do_pmda
     then
@@ -1093,7 +1094,7 @@ _remove()
 	if pmnsdel -n $PMNSROOT $__n >$tmp 2>&1
 	then
 	    rm -f $PMNSDIR/$__n
-	    $PCP_KILLALL_PROG -HUP pmcd >/dev/null 2>&1
+	    pmsignal -a -s HUP pmcd >/dev/null 2>&1
 	    echo "done"
 	else
 	    if grep 'Non-terminal "'"$__n"'" not found' $tmp >/dev/null
@@ -1167,7 +1168,7 @@ _check_userroot()
 {
     case "$PCP_PLATFORM"
     in
-	cygwin|mingw)
+	mingw)
 	    # nothing we can usefully do here, skip the test
 	    #
 	    ;;
