@@ -54,8 +54,6 @@ static time_t   	configFileTime;
 !bozo!
 #endif
 
-AgentInfo	*agent = NULL; /* List of agent info structs */
-int		nAgents = 0;		/* Number of structs in use */
 int		szAgents = 0;		/* Number currently allocated */
 int		mapdom[MAXDOMID+2];	/* The DomainId-to-AgentIndex map */
 					/* Don't use it during parsing! */
@@ -115,6 +113,22 @@ CopyToken(void)
     return copy;
 }
 
+/* Return a strdup-ed copy of the current token, plus an optional prefix. */
+static char*
+CopyPathToken(const char *prefix)
+{
+    int		len = (int)(tokenend - token);
+    int		plen = prefix ? strlen(prefix) : 0;
+    char	*copy = (char *)malloc(plen + len + 1);
+    if (copy != NULL) {
+	if (plen > 0)
+	    strncpy(copy, prefix, plen);
+	strncpy(copy + plen, token, len);
+	copy[plen + len] = '\0';
+    }
+    return copy;
+}
+
 /* Get the next line from the input stream into linebuf. */
 
 static void
@@ -512,8 +526,12 @@ BuildArgv(void)
     do {
 	/* Make result big enough for new arg and terminating NULL pointer */
 	result = (char **)realloc(result, (nArgs + 2) * sizeof(char *));
-	if (result != NULL)
-	    result[nArgs] = CopyToken();
+	if (result != NULL) {
+	    if (*token != '/')
+		result[nArgs] = CopyToken();
+	    else if ((result[nArgs] = CopyPathToken(getenv("PCP_DIR"))))
+		__pmNativePath(result[nArgs]);
+	}
 	if (result == NULL || result[nArgs] == NULL) {
 	    fprintf(stderr, "pmcd config: line %d, error building argument list\n",
 		    nLines);
@@ -597,28 +615,6 @@ FreeAgent(AgentInfo *ap)
     }
 }
 
-#ifdef __host_mips
-/*
- * Use compilation flags to determine what format this pmcd is compiled into.
- * pmcd can only load DSOs compiled in the same MIPS format that it is.
- *
- * Each of the MIPS executable/DSO formats has a different prefix to allow
- * multiple versions of the same DSO agent to coexist in one directory.
- */
-
-static char	*dsoPrefix =
-#if   _MIPS_SIM == _MIPS_SIM_ABI32
-    "mips_o32.";
-#elif _MIPS_SIM == _MIPS_SIM_NABI32
-    "mips_n32.";
-#elif _MIPS_SIM == _MIPS_SIM_ABI64
-    "mips_64.";
-#else
-    !!! bozo : dont know which MIPS executable format pmcd should be!!!
-#endif
-#endif /* __host_mips */
-
-
 /* Parse a DSO specification, creating and initialising a new entry in the
  * agent table if the spec has no errors.
  */
@@ -635,58 +631,28 @@ ParseDso(char *pmDomainLabel, int pmDomainId)
 	fprintf(stderr, "pmcd: line %d, expected DSO entry point\n", nLines);
 	return -1;
     }
-    else
-	if ((entryPoint = CopyToken()) == NULL) {
-	    fprintf(stderr,
-			 "pmcd config: line %d, couldn't copy DSO entry point\n",
+    if ((entryPoint = CopyToken()) == NULL) {
+	fprintf(stderr, "pmcd config: line %d, couldn't copy DSO entry point\n",
 			 nLines);
-	    __pmNoMem("pmcd config", tokenend - token + 1, PM_FATAL_ERR);
-	}
+	__pmNoMem("pmcd config", tokenend - token + 1, PM_FATAL_ERR);
+    }
 
     FindNextToken();
     if (*token == '\n') {
 	fprintf(stderr, "pmcd: line %d, expected DSO pathname\n", nLines);
 	return -1;
     }
-
-    if ((pathName = CopyToken()) == NULL) {
-	fprintf(stderr,
-		     "pmcd config: line %d, couldn't copy DSO pathname\n",
-		     nLines);
-	__pmNoMem("pmcd config", tokenend - token + 1, PM_FATAL_ERR);
-    }
-
-#ifdef __host_mips
-    /*
-     * if the DSO pathname is not absolute, prepend the appropriate "well
-     * known directory" where the DSOs have the same MIPS format as this
-     * pmcd executable.
-     */
-    if (*pathName != '/' && strcmp(pathName, "-") != 0) {
-	char	*newPath;
-	size_t	need = strlen(pathName) + strlen(dsoPrefix) + 1;
-
-	if ((newPath = (char *)malloc(need)) == NULL) {
-	    fprintf(stderr,
-			 "pmcd config: line %d, couldn't construct DSO pathname\n",
-			 nLines);
-	    __pmNoMem("pmcd config", need, PM_FATAL_ERR);
-	}
-	strcpy(newPath, dsoPrefix);
-	strcat(newPath, pathName);
-	free(pathName);
-	pathName = newPath;
-	xlatePath = 1;
-    }
-#else /* __host_mips */
-    /*
-     * Only absolute DSO pathnames are supported.
-     */
-    if (*pathName != '/') {
-	fprintf(stderr, "pmcd: line %d IGNORED : path \"%s\" to PMDA is not absolute\n", nLines, pathName);
+    if (*token != '/') {
+	fprintf(stderr, "pmcd: line %d IGNORED : path \"%s\" to PMDA is not absolute\n", nLines, token);
 	return -1;
     }
-#endif /* __host_mips */
+
+    if ((pathName = CopyPathToken(getenv("PCP_DIR"))) == NULL) {
+	fprintf(stderr, "pmcd config: line %d, couldn't copy DSO pathname\n",
+			nLines);
+	__pmNoMem("pmcd config", tokenend - token + 1, PM_FATAL_ERR);
+    }
+    __pmNativePath(pathName);
 
     FindNextToken();
     if (*token != '\n') {
@@ -809,11 +775,9 @@ ParsePipe(char *pmDomainLabel, int pmDomainId)
     FindNextToken();
     if (TokenIs("binary"))
 	pduProtocol = PDU_BINARY;
-    else if (TokenIs("text")) 
-	pduProtocol = PDU_ASCII;
     else {
 	fprintf(stderr,
-		     "pmcd: line %d, pipe PDU type expected (`binary' or `text')\n",
+		     "pmcd: line %d, pipe PDU type expected (`binary')\n",
 		     nLines);
 	return -1;
     }
@@ -1453,18 +1417,18 @@ CreateAgent(AgentInfo *aPtr)
     int		inPipe[2];	/* Pipe for input to child */
     int		outPipe[2];	/* For output to child */
     pid_t	childPid;
-    char	**argv;
+    char	**argv = NULL;
 
     if (aPtr->ipcType == AGENT_PIPE) {
 	argv = aPtr->ipc.pipe.argv;
-	if (pipe(inPipe) < 0) {
+	if (pipe1(inPipe) < 0) {
 	    fprintf(stderr,
 			 "pmcd: input pipe create failed for \"%s\" agent: %s\n",
 			 aPtr->pmDomainLabel, strerror(errno));
 	    return -1;
 	}
 
-	if (pipe(outPipe) < 0) {
+	if (pipe1(outPipe) < 0) {
 	    fprintf(stderr,
 			 "pmcd: output pipe create failed for \"%s\" agent: %s\n",
 			 aPtr->pmDomainLabel, strerror(errno));
@@ -1695,7 +1659,7 @@ GetAgentDso(AgentInfo *aPtr)
 #else
     fprintf(stderr, "Error attaching %s DSO at \"%s\"\n",
 		     aPtr->pmDomainLabel, dso->pathName);
-    fprintf(stderr, "No dynamic DSO/DLL support on this platform\n");
+    fprintf(stderr, "No dynamic shared library support on this platform\n");
     return -1;
 #endif
 
@@ -1821,7 +1785,8 @@ GetAgentDso(AgentInfo *aPtr)
 static void
 ContactAgents(void)
 {
-    int		i, sts;
+    int		i;
+    int		sts = 0;
     int		createdSocketAgents = 0;
     AgentInfo	*aPtr;
 
@@ -1965,41 +1930,12 @@ AgentsDiffer(AgentInfo *a1, AgentInfo *a2)
     if (a1->ipcType == AGENT_DSO) {
 	DsoInfo	*dso1 = &a1->ipc.dso;
 	DsoInfo	*dso2 = &a2->ipc.dso;
-#ifdef __host_mips 
-	char	*p1;
-	char	*p2;
-	int	pfxlen = (int)strlen(dsoPrefix);
-
-	if (dso1 == NULL || dso2 == NULL)
-		return 1;	/* should never happen */
-	if (dso1->pathName == NULL || dso2->pathName == NULL)
-		return 1;	/* should never happen */
-	p1 = dso1->pathName;
-	if (dso1->xlatePath) {
-	    p1 = basename(p1);
-	    if (strncmp(p1, dsoPrefix, pfxlen) == 0)
-		p1 += pfxlen;
-	}
-	p2 = dso2->pathName;
-	if (dso2->xlatePath) {
-	    p2 = basename(p2);
-	    if (strncmp(p2, dsoPrefix, pfxlen) == 0)
-		p2 += pfxlen;
-	}
-	if (strcmp(p1, p2))
-	    return 1;
-	if (dso1->entryPoint == NULL || dso2->entryPoint == NULL)
-		return 1;	/* should never happen */
-	if (strcmp(dso1->entryPoint, dso2->entryPoint))
-	    return 1;
-#else /* __host_mips */
 	if (strcmp(dso1->pathName, dso2->pathName) != 0)
 	    return 1;
 	if (dso1->entryPoint == NULL || dso2->entryPoint == NULL)
-		return 1;	/* should never happen */
+	    return 1;	/* should never happen */
 	if (strcmp(dso1->entryPoint, dso2->entryPoint))
 	    return 1;
-#endif
     }
     else if (a1->ipcType == AGENT_SOCKET) {
 	SocketInfo	*sock1 = &a1->ipc.socket;
