@@ -314,6 +314,7 @@ jstat_command(int pid)
 {
     size_t size;
     char *command;
+    int sep = __pmPathSeparator();
 
     size = (jstat_path == NULL) ? 0 : (strlen(jstat_path) + 1);
     size += strlen(JSTAT_COMMAND) + 32;
@@ -322,7 +323,7 @@ jstat_command(int pid)
     if (jstat_path == NULL)
 	sprintf(command, "%s%u", JSTAT_COMMAND, pid);
     else
-	sprintf(command, "%s/%s%u", jstat_path, JSTAT_COMMAND, pid);
+	sprintf(command, "%s%c%s%u", jstat_path, sep, JSTAT_COMMAND, pid);
     return command;
 }
 
@@ -388,6 +389,7 @@ void
 jstat_indom_check(void)
 {
     FILE *pp;
+    int sep = __pmPathSeparator();
     static int initialised;
     static char jps[BUFFER_MAXLEN];
 
@@ -395,7 +397,7 @@ jstat_indom_check(void)
 	if (jstat_path == NULL)
 	    strcpy(jps, "jps");
 	else
-	    sprintf(jps, "%s/jps", jstat_path);
+	    sprintf(jps, "%s%cjps", jstat_path, sep);
 	initialised = 1;
     }
 
@@ -638,16 +640,17 @@ int
 main(int argc, char **argv)
 {
     int			err = 0;
+    int			sep = __pmPathSeparator();
     char		*endnum;
     pmdaInterface	dispatch;
     int			c;
-    char		helptext[MAXPATHLEN];
+    char		path[MAXPATHLEN];
 
     __pmSetProgname(argv[0]);
-    snprintf(helptext, sizeof(helptext), "%s/jstat/help",
-		pmGetConfig("PCP_PMDAS_DIR"));
+    snprintf(path, sizeof(path), "%s%c" "jstat" "%c" "help",
+		pmGetConfig("PCP_PMDAS_DIR"), sep, sep);
     pmdaDaemon(&dispatch, PMDA_INTERFACE_3, pmProgname, JSTAT,
-		"jstat.log", helptext);
+		"jstat.log", path);
 
     while ((c = pmdaGetOpt(argc, argv, "D:d:h:i:l:pu:" "H:J:P:r:?",
 			   &dispatch, &err)) != EOF) {
@@ -685,11 +688,17 @@ main(int argc, char **argv)
 	usage();
     else if ((argc - optind) == 1)
 	jstat_pcp_dir_name = argv[optind];
-    else
-	jstat_pcp_dir_name = "/var/tmp/jstat";
+    else {
+	snprintf(path, sizeof(path), "%s%c" "jstat",
+		 pmGetConfig("PCP_TMP_DIR"), sep);
+	jstat_pcp_dir_name = path;
+    }
 
-    if (java_home)
-	setenv("JAVA_HOME", java_home, 1);
+    if (java_home) {
+	char home[MAXPATHLEN + 16];
+	snprintf(home, sizeof(home), "JAVA_HOME=%s", __pmNativePath(java_home));
+	putenv(home);
+    }
 
     pmdaOpenLog(&dispatch);
     jstat_init(&dispatch);
