@@ -14,16 +14,12 @@
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
- *
- * Contact information: Silicon Graphics, Inc., 1500 Crittenden Lane,
- * Mountain View, CA 94043, USA, or: http://www.sgi.com
  */
 
 /*
  * This code contributed by Mike Mason (mmlnx@us.ibm.com)
- * $Id: ksym.h,v 1.6 2004/06/24 06:15:36 kenmcd Exp $
  */
-#include "platform_defs.h"
+#define KERNEL_VERSION(a,b,c) (((a) << 16) + ((b) << 8) + (c))
 
 #define INIT_KSIZE 8192
 #define INCR_KSIZE 2048
