@@ -14,17 +14,12 @@
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
- * 
- * Contact information: Silicon Graphics, Inc., 1500 Crittenden Lane,
- * Mountain View, CA 94043, USA, or: http://www.sgi.com
  */
 
 /*
  * This code contributed by Michal Kara (lemming@arthur.plbohnice.cz)
  */
 
-#ident "$Id: proc_net_tcp.h,v 1.2 2004/06/24 06:15:36 kenmcd Exp $"
- 
 enum {
 	_PM_TCP_ESTABLISHED = 1,
 	_PM_TCP_SYN_SENT,
