@@ -23,18 +23,6 @@
 #include "pmcd/src/client.h"
 #include "pmie/src/pmiestats.h"
 #include <sys/stat.h>
-#if defined(HAVE_SYS_MMAN_H)
-#include <sys/mman.h>
-#endif
-
-/*
- * To debug with dbpmda, add this so the symbols from pmcd(1) will
- * be faked out:
- *
- * #define DEBUG_WITH_DBPMDA
- */
-
-#define PMCD			2
 
 /*
  * Note: strange numbering for pmcd.pdu_{in,out}.total for
@@ -198,6 +186,12 @@ static pmDesc	desctab[] = {
     { PMDA_PMID(5,8), PM_TYPE_FLOAT, PM_INDOM_NULL, PM_SEM_DISCRETE, PMDA_PMUNITS(0,-1,1,0,PM_TIME_SEC,PM_COUNT_ONE) },
 /* pmie.eval.actual */
     { PMDA_PMID(5,9), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) },
+
+/* client.whoami */
+    { PMDA_PMID(6,0), PM_TYPE_STRING, PM_INDOM_NULL, PM_SEM_DISCRETE, PMDA_PMUNITS(0,0,0,0,0,0) },
+/* client.start_date */
+    { PMDA_PMID(6,1), PM_TYPE_STRING, PM_INDOM_NULL, PM_SEM_DISCRETE, PMDA_PMUNITS(0,0,0,0,0,0) },
+
 /* End-of-List */
     { PM_ID_NULL, 0, 0, 0, PMDA_PMUNITS(0, 0, 0, 0, 0, 0) }
 };
@@ -216,6 +210,8 @@ static pmInDom		pmdaindom;
 static pmInDom		pmieindom;
 #define INDOM_POOL	5
 static pmInDom		bufindom;
+#define INDOM_CLIENT	6
+static pmInDom		clientindom;
 
 #define NUMREG 16
 static int		reg[NUMREG];
@@ -224,9 +220,6 @@ typedef struct {
     pid_t	pid;
     int		size;
     char	*name;
-#ifdef IS_MINGW
-    HANDLE	hdl;
-#endif
     void	*mmap;
 } pmie_t;
 static pmie_t		*pmies;
@@ -246,6 +239,14 @@ static struct {
 };
 static int	nbufsz = sizeof(bufinst) / sizeof(bufinst[0]);
 
+typedef struct {
+    int		id;		/* index into client[] */
+    int		seq;
+    char	*value;
+} whoami_t;
+static whoami_t		*whoamis;
+static unsigned int	nwhoamis;
+
 /*
  * this routine is called at initialization to patch up any parts of the
  * desctab that cannot be statically initialized, and to optionally
@@ -279,6 +280,10 @@ init_tables(int dom)
     indomp->pad = 0;
     indomp->domain = dom;
     indomp->serial = INDOM_POOL;
+    indomp = (__pmInDom_int *)&clientindom;
+    indomp->pad = 0;
+    indomp->domain = dom;
+    indomp->serial = INDOM_CLIENT;
 
     /* merge performance domain id part into PMIDs in pmDesc table */
     for (i = 0; desctab[i].pmid != PM_ID_NULL; i++) {
@@ -294,6 +299,8 @@ init_tables(int dom)
 	    desctab[i].indom = pmdaindom;
 	else if (pmidp->cluster == 5)
 	    desctab[i].indom = pmieindom;
+	else if (pmidp->cluster == 6)
+	    desctab[i].indom = clientindom;
     }
     ndesc--;
 }
@@ -313,12 +320,7 @@ remove_pmie_indom(void)
 
     for (n = 0; n < npmies; n++) {
 	free(pmies[n].name);
-#ifdef IS_MINGW
-	UnmapViewOfFile(pmies[n].mmap);
-	CloseHandle(pmies[n].hdl);
-#else /* POSIX */
-	munmap(pmies[n].mmap, pmies[n].size);
-#endif
+	__pmMemoryUnmap(pmies[n].mmap, pmies[n].size);
     }
     free(pmies);
     pmies = NULL;
@@ -339,9 +341,7 @@ refresh_pmie_indom(void)
     void		*ptr;
     DIR			*pmiedir;
     int			fd;
-#ifdef IS_MINGW
-    HANDLE		hdl;
-#endif
+    int			sep = __pmPathSeparator();
 
     if (stat(PMIE_DIR, &statbuf) == 0) {
 #if defined(HAVE_ST_MTIME_WITH_E) && defined(HAVE_STAT_TIME_T)
@@ -376,7 +376,7 @@ refresh_pmie_indom(void)
 		    continue;
 		if (!__pmProcessExists(pmiepid))
 		    continue;
-		snprintf(fullpath, sizeof(fullpath), "%s/%s", PMIE_DIR, dp->d_name);
+		snprintf(fullpath, sizeof(fullpath), "%s%c%s", PMIE_DIR, sep, dp->d_name);
 		if (stat(fullpath, &statbuf) < 0) {
 		    __pmNotifyErr(LOG_WARNING, "pmcd pmda cannot stat %s: %s",
 				fullpath, strerror(errno));
@@ -399,39 +399,18 @@ refresh_pmie_indom(void)
 		    free(endp);
 		    continue;
 		}
-#ifdef IS_MINGW
-		hdl = CreateFileMapping((HANDLE)_get_osfhandle(fd),
-				NULL, PAGE_READONLY, 0, statbuf.st_size, NULL);
-		if (hdl == INVALID_HANDLE_VALUE) {
-		    __pmNotifyErr(LOG_ERR, "pmcd pmda mapping of %s failed: %s",
-				fullpath, strerror(errno));
-		    free(endp);
-		    close(fd);
-		    continue;
-		}
-		ptr = MapViewOfFile(hdl, FILE_MAP_READ, 0, 0, statbuf.st_size);
-#else /* POSIX */
-		ptr = mmap(NULL, statbuf.st_size, PROT_READ, MAP_SHARED, fd, 0);
-#endif
+		ptr = __pmMemoryMap(fd, statbuf.st_size, 0);
 		close(fd);
-		if (ptr == MAP_FAILED) {
-		    __pmNotifyErr(LOG_ERR, "pmcd pmda mmap of %s failed: %s",
+		if (ptr == NULL) {
+		    __pmNotifyErr(LOG_ERR, "pmcd pmda memmap of %s failed: %s",
 				fullpath, strerror(errno));
-#ifdef IS_MINGW
-		    CloseHandle(hdl);
-#endif
 		    free(endp);
 		    continue;
 		}
 		else if (((pmiestats_t *)ptr)->version != 1) {
 		    __pmNotifyErr(LOG_WARNING, "incompatible pmie version: %s",
 				fullpath);
-#ifdef IS_MINGW
-		    UnmapViewOfFile(ptr);
-		    CloseHandle(hdl);
-#else
-		    munmap(ptr, statbuf.st_size);
-#endif
+		    __pmMemoryUnmap(ptr, statbuf.st_size);
 		    free(endp);
 		    continue;
 		}
@@ -439,9 +418,6 @@ refresh_pmie_indom(void)
 		pmies[npmies].name = endp;
 		pmies[npmies].size = statbuf.st_size;
 		pmies[npmies].mmap = ptr;
-#ifdef IS_MINGW
-		pmies[npmies].hdl  = hdl;
-#endif
 		npmies++;
 	    }
 	    closedir(pmiedir);
@@ -494,7 +470,7 @@ pmcd_instance_reg(int inst, char *name, __pmInResult **result)
 	/* return inst and name for everything */
 	for (i = 0; i < res->numinst; i++) {
 	    res->instlist[i] = i;
-	    sprintf(idx, "%d", i);
+	    snprintf(idx, sizeof(idx), "%d", i);
 	    if ((res->namelist[i] = strdup(idx)) == NULL) {
 		__pmFreeInResult(res);
 		return -errno;
@@ -504,7 +480,7 @@ pmcd_instance_reg(int inst, char *name, __pmInResult **result)
     else if (name == NULL) {
 	/* given an inst, return the name */
 	if (0 <= inst && inst < NUMREG) {
-	    sprintf(idx, "%d", inst);
+	    snprintf(idx, sizeof(idx), "%d", inst);
 	    if ((res->namelist[0] = strdup(idx)) == NULL) {
 		__pmFreeInResult(res);
 		return -errno;
@@ -616,17 +592,17 @@ pmcd_instance(pmInDom indom, int inst, char *name, __pmInResult **result, pmdaEx
     int			sts = 0;
     __pmInResult	*res;
     int			getall = 0;
-    int			getname;
-    int			nports;
+    int			getname = 0;	/* initialize to pander to gcc */
+    int			nports = 0;	/* initialize to pander to gcc */
     __pmLogPort		*ports;
-    unsigned int	pmiecount;
+    unsigned int	pmiecount = 0;	/* initialize to pander to gcc */
     int			i;
 
     if (indom == regindom)
 	return pmcd_instance_reg(inst, name, result);
     else if (indom == bufindom)
 	return pmcd_instance_pool(inst, name, result);
-    else if (indom == logindom || indom == pmdaindom || indom == pmieindom) {
+    else if (indom == logindom || indom == pmdaindom || indom == pmieindom || indom == clientindom) {
 	res = (__pmInResult *)malloc(sizeof(__pmInResult));
 	if (res == NULL)
 	    return -errno;
@@ -653,10 +629,16 @@ pmcd_instance(pmInDom indom, int inst, char *name, __pmInResult **result, pmdaEx
 
 	    if (indom == logindom)
 		res->numinst = nports;
+	    else if (indom == pmdaindom)
+		res->numinst = nAgents;
 	    else if (indom == pmieindom)
 		res->numinst = pmiecount;
-	    else
-		res->numinst = nAgents;
+	    else if (indom == clientindom) {
+		res->numinst = 0;
+		for (i = 0; i < nClients; i++) {
+		    if (client[i].status.connected) res->numinst++;
+		}
+	    }
 	}
 	else {
 	    getname = name == NULL;
@@ -776,7 +758,7 @@ pmcd_instance(pmInDom indom, int inst, char *name, __pmInResult **result, pmdaEx
 		res->instlist[0] = pmies[i].pid;
 	}
     }
-    else {
+    else if (indom == pmdaindom) {
 	res->indom = pmdaindom;
 
 	if (getall) {		/* get instance ids and names */
@@ -822,6 +804,64 @@ pmcd_instance(pmInDom indom, int inst, char *name, __pmInResult **result, pmdaEx
 		res->instlist[0] = agent[i].pmDomainId;
 	}
     }
+    else if (indom == clientindom) {
+	res->indom = clientindom;
+
+	if (getall) {		/* get instance ids and names */
+	    int		k = 0;
+	    for (i = 0; i < nClients; i++) {
+		char	buf[11];	/* enough for 32-bit client seq number */
+		if (!client[i].status.connected)
+		    continue;
+		res->instlist[k] = client[i].seq;
+		snprintf(buf, sizeof(buf), "%u", client[i].seq);
+		res->namelist[k] = strdup(buf);
+		if (res->namelist[k] == NULL) {
+		    sts = -errno;
+		    __pmNoMem("pmcd_instance pmGetInDom",
+			     strlen(buf), PM_RECOV_ERR);
+		    /* ensure pmFreeInResult only gets valid pointers */
+		    res->numinst = i;
+		    break;
+		}
+		k++;
+	    }
+	}
+	else if (getname) {	/* given id, get name */
+	    for (i = 0; i < nClients; i++) {
+		if (client[i].status.connected && inst == client[i].seq)
+		    break;
+	    }
+	    if (i == nClients) {
+		sts = PM_ERR_INST;
+		res->namelist[0] = NULL;
+	    }
+	    else {
+		char	buf[11];	/* enough for 32-bit client seq number */
+		snprintf(buf, sizeof(buf), "%u", (unsigned int)inst);
+		res->namelist[0] = strdup(buf);
+		if (res->namelist[0] == NULL) {
+		    sts = -errno;
+		    __pmNoMem("pmcd_instance pmNameInDom",
+			     strlen(buf), PM_RECOV_ERR);
+		}
+	    }
+	}
+	else {			/* given name, get id */
+	    char	buf[11];	/* enough for 32-bit client seq number */
+	    for (i = 0; i < nClients; i++) {
+		if (!client[i].status.connected)
+		    continue;
+		snprintf(buf, sizeof(buf), "%u", client[i].seq);
+		if (strcmp(name, buf) == 0)
+		    break;
+	    }
+	    if (i == nClients)
+		sts = PM_ERR_INST;
+	    else
+		res->instlist[0] = client[i].seq;
+	}
+    }
 
     if (sts < 0) {
 	__pmFreeInResult(res);
@@ -879,12 +919,12 @@ simabi()
 # else
     return SIM_ABI;	/* SIM_ABI is defined in the linux Makefile */
 # endif /* __linux__ */
-#elif defined(IS_SOLARIS) || defined(IS_INTERIX) || defined(IS_FREEBSD)
+#elif defined(IS_SOLARIS) || defined(IS_FREEBSD)
     return "elf";
 #elif defined(IS_DARWIN)
     return "Mach-O";
-#elif defined(IS_CYGWIN) || defined(IS_MINGW)
-    return "i386";
+#elif defined(IS_MINGW)
+    return "i386";	// TODO: need to handle x86_64 too
 #elif defined(IS_AIX)
     return "powerpc";
 #else
@@ -907,7 +947,7 @@ pmcd_fetch(int numpmid, pmID pmidlist[], pmResult **resp, pmdaExt *pmda)
     static char		*hostname = NULL;
     pmiestats_t		*pmie;
     pmValueSet		*vset;
-    pmDesc		*dp;
+    pmDesc		*dp = NULL;	/* initialize to pander to gcc */
     __pmID_int		*pmidp;
     pmAtomValue		atom;
     __pmLogPort		*lpp;
@@ -1210,8 +1250,6 @@ pmcd_fetch(int numpmid, pmID pmidlist[], pmResult **resp, pmdaExt *pmda)
 		    switch (pmidp->item) {
 			case 0:		/* agent.type */
 			    atom.ul = agent[j].ipcType << 1;
-			    if (agent[j].pduProtocol == PDU_ASCII)
-				atom.ul |= 1;
 			    break;
 			case 1:		/* agent.status */
 			    if (agent[j].status.notReady)
@@ -1301,6 +1339,60 @@ pmcd_fetch(int numpmid, pmID pmidlist[], pmResult **resp, pmdaExt *pmda)
 		    pmSortInstances(&sortme);
 		}
 		break;
+
+	    case 6:	/* client metrics */
+		for (j = numval = 0; j < nClients; j++) {
+		    if (!client[j].status.connected)
+			continue;
+		    if (__pmInProfile(clientindom, _profile, client[j].seq))
+			numval++;
+		}
+		if (numval != 1) {
+		    /* need a different vset size */
+		    if (vset_resize(res, i, 1, numval) == -1)
+			return -ENOMEM;
+		    vset = res->vset[i];
+		    vset->pmid = pmidlist[i];
+		}
+		for (j = numval = 0; j < nClients; ++j) {
+		    int		k;
+		    if (!client[j].status.connected)
+			continue;
+		    if (!__pmInProfile(clientindom, _profile, client[j].seq))
+			continue;
+		    vset->vlist[numval].inst = client[j].seq;
+		    switch (pmidp->item) {
+			case 0:		/* client.whoami */
+			    for (k = 0; k < nwhoamis; k++) {
+				if (whoamis[k].seq == client[j].seq) {
+				    atom.cp = strdup(whoamis[k].value);
+				    break;
+				}
+			    }
+			    if (k == nwhoamis)
+				/* no id registered, so no value */
+				atom.cp = strdup("");
+			    break;
+			case 1:		/* client.start_date */
+			    atom.cp = strdup(ctime(&client[j].start));
+			    /* trim trailing \n */
+			    k = strlen(atom.cp);
+			    atom.cp[k-1] = '\0';
+			    break;
+		    }
+		    if ((sts = __pmStuffValue(&atom, 0,
+				&vset->vlist[numval], dp->type)) < 0)
+			break;
+		    valfmt = sts;
+		    numval++;
+		}
+		if (numval > 0) {
+		    pmResult	sortme;
+		    sortme.numpmid = 1;
+		    sortme.vset[0] = vset;
+		    pmSortInstances(&sortme);
+		}
+		break;
 	}
 
 	if (sts == 0 && valfmt == -1 && vset->numval == 1)
@@ -1436,6 +1528,63 @@ pmcd_store(pmResult *result, pmdaExt *pmda)
 		break;
 	    }
 	}
+	else if (pmidp->cluster == 6) {
+	    if (pmidp->item == 0) {	/* pmcd.client.whoami */
+		/*
+		 * Expect one value for one instance (PM_IN_NULL)
+		 *
+		 * Use the value from the pmResult to change the value
+		 * for the client[] that matches the current pmcd client.
+		 */
+		char	*cp = vsp->vlist[0].value.pval->vbuf;
+		int	j;
+		int	last_free = -1;
+
+		if (vsp->numval != 1 || vsp->vlist[0].inst != PM_IN_NULL) {
+		    return PM_ERR_INST;
+		}
+		for (j = 0; j < nwhoamis; j++) {
+		    if (whoamis[j].id == -1) {
+			/* slot in whoamis[] not in use */
+			last_free = j;
+			continue;
+		    }
+		    if (whoamis[j].id == this_client_id &&
+		        whoamis[j].seq == client[this_client_id].seq) {
+			/* found the one to replace */
+			free(whoamis[j].value);
+			break;
+		    }
+		    if (!client[whoamis[j].id].status.connected ||
+		        client[whoamis[j].id].seq != whoamis[j].seq) {
+			/* old whoamis[] entry, mark as available for reuse */
+			free(whoamis[j].value);
+			whoamis[j].id = -1;
+			last_free = j;
+		    }
+		}
+		if (j == nwhoamis) {
+		    if (last_free != -1) {
+			j = last_free;
+		    }
+		    else {
+			nwhoamis++;
+			if ((whoamis = (whoami_t *)realloc(whoamis, nwhoamis*sizeof(whoamis[0]))) == NULL) {
+			    __pmNoMem("pmstore whoami", nwhoamis*sizeof(whoamis[0]), PM_RECOV_ERR);
+			    nwhoamis = 0;
+			    return -ENOMEM;
+			}
+		    }
+		    whoamis[j].id = this_client_id;
+		    whoamis[j].seq = client[this_client_id].seq;
+		}
+		whoamis[j].value = strdup(cp);
+	    }
+	    else {
+		sts = PM_ERR_PMID;
+		break;
+	    }
+	}
 	else {
 	    /* not one of the metrics we are willing to change */
 	    sts = PM_ERR_PMID;
@@ -1450,9 +1599,10 @@ void
 pmcd_init(pmdaInterface *dp)
 {
     char helppath[MAXPATHLEN];
+    int sep = __pmPathSeparator();
  
-    snprintf(helppath, sizeof(helppath), "%s/pmdas/pmcd/help",
-	 	pmGetConfig("PCP_VAR_DIR"));
+    snprintf(helppath, sizeof(helppath), "%s%c" "pmcd" "%c" "help",
+		pmGetConfig("PCP_PMDAS_DIR"), sep, sep);
     pmdaDSO(dp, PMDA_INTERFACE_2, "pmcd", helppath);
 
     dp->version.two.profile = pmcd_profile;
@@ -1465,51 +1615,3 @@ pmcd_init(pmdaInterface *dp)
 
     pmdaInit(dp, NULL, 0, NULL, 0);
 }
-
-#ifdef DEBUG_WITH_DBPMDA
-/*
- * from pmcd's address space  ... weak symbols added here for use
- * with dbpmda
- */
-int		_pmcd_done;
-#pragma weak _pmcd_done
-char		*_pmcd_data;
-#pragma weak _pmcd_data
-int		_pmcd_timeout;
-#pragma weak _pmcd_timeout
-int		_pmcd_trace_nbufs;
-#pragma weak _pmcd_trace_nbufs
-int		_pmcd_trace_mask;
-#pragma weak _pmcd_trace_mask
-int		nAgents;
-#pragma weak nAgents
-AgentInfo	*agent;
-#pragma weak agent
-int		nClients;
-#pragma weak nClients
-ClientInfo	*client;
-#pragma weak client
-int		pmcd_hi_openfds = -1;
-#pragma weak pmcd_hi_openfds
-
-void pmcd_init_trace(int n)
-{
-    fprintf(stderr, "dummy pmcd_init_trace(%d) called\n", n);
-}
-#pragma weak pmcd_init_trace
-
-void
-pmcd_dump_trace(FILE *f)
-{
-    fprintf(f, "dummy pmcd_dump_trace called\n");
-}
-#pragma weak pmcd_dump_trace
-
-void
-ShowClients(FILE *f)
-{
-    fprintf(f, "dummy ShowClients called\n");
-}
-#pragma weak ShowClients
-
-#endif /* DEBUG_WITH_DBPMDA */
