@@ -447,6 +447,7 @@ main(int argc, char **argv)
     int			argCount = 0;
     int			checkOnly = 0;
     int			sts = 0;
+    int			sep = __pmPathSeparator();
     int			n = 0;
     int			serverTableSize = 0;
     int			regexTableSize = 0;
@@ -476,8 +477,8 @@ main(int argc, char **argv)
 
     wl_isDSO = 0;
 
-    snprintf(wl_helpFile, sizeof(wl_helpFile),
-		"%s/weblog/help", pmGetConfig("PCP_PMDAS_DIR"));
+    snprintf(wl_helpFile, sizeof(wl_helpFile), "%s%c" "weblog" "%c" "help",
+		pmGetConfig("PCP_PMDAS_DIR"), sep, sep);
     pmdaDaemon(&desc, PMDA_INTERFACE_2, pmProgname, WEBSERVER,
 		wl_logFile, wl_helpFile);
 
@@ -1078,7 +1079,7 @@ main(int argc, char **argv)
 	for (n=1; n<=wl_numSprocs; n++) {
 	    proc = &wl_sproc[n];
 
-	    sts = pipe(proc->inFD);
+	    sts = pipe1(proc->inFD);
 	    if (sts) {
 	    	logmessage(LOG_ERR, 
 			     "Cannot allocate fileDes 1 for sproc[%d]", 
@@ -1095,7 +1096,7 @@ main(int argc, char **argv)
 			      n);
 #endif
 
-	    sts = pipe(proc->outFD);
+	    sts = pipe1(proc->outFD);
 	    if (sts) {
 	    	logmessage(LOG_ERR, 
 			     "Cannot allocate fileDes 2 for sproc[%d]", 
