@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008 Aconex.  All Rights Reserved.
+ * Copyright (c) 2008-2009 Aconex.  All Rights Reserved.
  * Copyright (c) 2004 Silicon Graphics, Inc.  All Rights Reserved.
  * Parts of this file contributed by Ken McDonell
  * (kenj At internode DoT on DoT net)
@@ -13,59 +13,34 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  */
 #include "hypnotoad.h"
 #include <ctype.h>
 
-pmInDom
-windows_indom(int qid, int domain)
-{
-    pmInDom	result;
-    int		serial;
-
-    switch (qid) {
-    case Q_DISK_DEV:		serial = DISK_INDOM; break;
-    case Q_PERCPU:		serial = CPU_INDOM; break;
-    case Q_NETWORK_IF:		serial = NETIF_INDOM; break;
-    case Q_LDISK:		serial = FILESYS_INDOM; break;
-    case Q_SQLSERVER_LOCK:	serial = SQL_LOCK_INDOM; break;
-    case Q_SQLSERVER_CACHE:	serial = SQL_CACHE_INDOM; break;
-    case Q_SQLSERVER_DB:	serial = SQL_DB_INDOM; break;
-    case Q_PROCESSES:		serial = PROCESS_INDOM; break;
-    case Q_THREADS:		serial = THREAD_INDOM; break;
-    default:			return PM_INDOM_NULL;
-    }
-    return INDOM(domain, serial);
-}
-
 void
 windows_instance_refresh(pmInDom indom)
 {
-    int		i;
+    int			i, setup;
+
+    setup = windows_indom_setup[pmInDom_serial(indom)];
 
     for (i = 0; i < metricdesc_sz; i++) {
 	pdh_metric_t *mp = &metricdesc[i];
 
 	if (indom != mp->desc.indom || mp->pat[0] == '\0')
 	    continue;
-	if (mp->flags & M_REDO) {
-	    pmdaCacheOp(indom, PMDA_CACHE_INACTIVE);
-	    windows_check_metric(mp);
-	}
+	if (!setup || (mp->flags & M_REDO))
+	    windows_visit_metric(mp, NULL);
 	break;
     }
 }
 
 int
-windows_check_instance(char *path, pdh_metric_t *mp)
+windows_lookup_instance(char *path, pdh_metric_t *mp)
 {
     __pmInDom_int	*ip;
     char		*p, *q, *name;
-    int			ok = 0;
+    int			sts, ok = 0;
 
     if (mp->desc.indom == PM_INDOM_NULL)
 	return PM_IN_NULL;
@@ -89,7 +64,6 @@ windows_check_instance(char *path, pdh_metric_t *mp)
 		     */
 		    return -1;
 		}
-		// inst = atoi(p);
 		while (isascii(*p) && isdigit(*p)) p++;
 		if (*p == ' ') {
 		    p++;
@@ -148,10 +122,9 @@ windows_check_instance(char *path, pdh_metric_t *mp)
 		    return -1;
 		}
 		int inst = atoi(p);
-		name = (char *)malloc(6);	// "cpuNN"
-		if (name != NULL) {
+		name = (char *)malloc(8);	// "cpuNNNN"
+		if (name != NULL)
 		    sprintf(name, "cpu%d", inst);
-		}
 		ok = 1;
 	    }
 	    /*
@@ -223,6 +196,10 @@ windows_check_instance(char *path, pdh_metric_t *mp)
 		     */
 		    return -1;
 		}
+		while (isascii(*p) && isdigit(*p))
+		    p++;
+		if (*p == ' ')
+		    p++;
 		q = strchr(p, ')');
 		if (q != NULL) {
 		    name = (char *)malloc(q - p + 1);
@@ -312,7 +289,8 @@ windows_check_instance(char *path, pdh_metric_t *mp)
 	    p = strchr(path, '(');	// skip hostname and Process/Thread
 	    if (p != NULL) {
 		p++;
-		if (strncmp(p, "_Total)", 7) == 0) {
+		if ((strncmp(p, "_Total)", 7) == 0) ||
+		    (strncmp(p, "_Total/", 7) == 0)) {
 		    /*
 		     * The totals are done as independent metrics,
 		     * just skip them here
@@ -353,5 +331,9 @@ windows_check_instance(char *path, pdh_metric_t *mp)
 	    return -1;
     }
 
-    return pmdaCacheStore(mp->desc.indom, PMDA_CACHE_ADD, name, NULL);
+    sts = pmdaCacheLookupName(mp->desc.indom, name, &ok, NULL);
+    if (sts != PMDA_CACHE_ACTIVE)
+	ok = pmdaCacheStore(mp->desc.indom, PMDA_CACHE_ADD, name, NULL);
+    free(name);
+    return ok;
 }
