@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008 Aconex.  All Rights Reserved.
+ * Copyright (c) 2008-2009 Aconex.  All Rights Reserved.
  * Copyright (c) 2004 Silicon Graphics, Inc.  All Rights Reserved.
  * Parts of this file contributed by Ken McDonell
  * (kenj At internode DoT on DoT net)
@@ -13,15 +13,19 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  */
 
 #include "hypnotoad.h"
 #include <winbase.h>
 
+#define roundup(x,y)	((((x) + ((y) - 1)) / (y)) * (y))
+
+char *windows_uname;
+char *windows_build;
+char *windows_machine;
+unsigned long windows_pagesize;
+int windows_indom_setup[NUMINDOMS];
+
 /*
  * This block of functionality is required to map counter types from
  * their Windows semantics to equivalent PCP semantics.
@@ -71,44 +75,19 @@ static struct {
 
 static int ctypetab_sz = sizeof(ctypetab) / sizeof(ctypetab[0]);
 
-char *
+static char *
 decode_ctype(DWORD ctype)
 {
     static char	unknown[20];
     int		i;
 
-    for (i = 0; i < ctypetab_sz; i++) {
-	if (ctype == ctypetab[i].type) {
+    for (i = 0; i < ctypetab_sz; i++)
+	if (ctype == ctypetab[i].type)
 	    return ctypetab[i].desc;
-	}
-    }
     sprintf(unknown, "0x%08x unknown", (int)ctype);
     return unknown;
 }
 
-/*
- * There has to be something like this hiding in the Windows DLLs,
- * but I cannot find it ... so roll your own.
- */
-void
-errmsg(void)
-{
-    LPVOID bufp;
-
-    FormatMessage(
-        FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
-        NULL,
-        GetLastError(),
-        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
-        (LPTSTR)&bufp,
-        0,
-	NULL);
-
-    fprintf(stderr, "%s\n", (const char *)bufp);
-
-    LocalFree(bufp);
-}
-
 static char *
 string_append(char *name, char *suff)
 {
@@ -160,47 +139,55 @@ _typestr(int type)
     }
 }
 
-char *
+#if 0	// debugging
+static char *
 _ctypestr(int ctype)
 {
     if (ctype == PERF_COUNTER_COUNTER)
 	return "PERF_COUNTER_COUNTER";
     else if (ctype == PERF_RAW_FRACTION)
 	return "PERF_RAW_FRACTION";
-    else if (ctype ==PERF_COUNTER_LARGE_RAWCOUNT_HEX)
+    else if (ctype == PERF_COUNTER_LARGE_RAWCOUNT_HEX)
 	return "PERF_COUNTER_LARGE_RAWCOUNT_HEX";
-    else if (ctype ==PERF_COUNTER_LARGE_RAWCOUNT)
+    else if (ctype == PERF_COUNTER_LARGE_RAWCOUNT)
 	return "PERF_COUNTER_LARGE_RAWCOUNT";
-    else if (ctype ==PERF_PRECISION_100NS_TIMER)
+    else if (ctype == PERF_PRECISION_100NS_TIMER)
 	return "PERF_PRECISION_100NS_TIMER";
-    else if (ctype ==PERF_100NSEC_TIMER)
+    else if (ctype == PERF_100NSEC_TIMER)
 	return "PERF_100NSEC_TIMER";
-    else if (ctype ==PERF_COUNTER_BULK_COUNT)
+    else if (ctype == PERF_COUNTER_BULK_COUNT)
 	return "PERF_COUNTER_BULK_COUNT";
-    else if (ctype ==PERF_COUNTER_RAWCOUNT_HEX)
+    else if (ctype == PERF_COUNTER_RAWCOUNT_HEX)
 	return "PERF_COUNTER_RAWCOUNT_HEX";
-    else if (ctype ==PERF_COUNTER_RAWCOUNT)
+    else if (ctype == PERF_COUNTER_RAWCOUNT)
 	return "PERF_COUNTER_RAWCOUNT";
-    else if (ctype ==PERF_COUNTER_COUNTER)
+    else if (ctype == PERF_COUNTER_COUNTER)
 	return "PERF_COUNTER_COUNTER";
     else
     	return "UNKNOWN";
 }
+#endif
 
 /*
  * Based on documentation from ...
  * http://msdn.microsoft.com/library/default.asp?
  * 		url=/library/en-us/sysinfo/base/osversioninfoex_str.asp
  */
-static char *
-format_uname(OSVERSIONINFOEX osv)
+static void
+windows_format_uname(OSVERSIONINFOEX osv)
 {
-    static char		*name = NULL;
     char		tbuf[80];
+    char		*name = NULL;
 
     switch (osv.dwPlatformId) {
         case VER_PLATFORM_WIN32_NT:
 	    if (osv.dwMajorVersion == 6 && osv.dwMinorVersion == 0) {
+		if (osv.wProductType == VER_NT_WORKSTATION)
+		    name = string_append(name, "Windows 7");
+		else
+		    name = string_append(name, "Windows Server 2008 R2");
+	    }
+	    else if (osv.dwMajorVersion == 6 && osv.dwMinorVersion == 0) {
 		if (osv.wProductType == VER_NT_WORKSTATION)
 		    name = string_append(name, "Windows Vista");
 		else
@@ -214,70 +201,67 @@ format_uname(OSVERSIONINFOEX osv)
 		name = string_append(name, "Windows 2000");
 	    else if (osv.dwMajorVersion <= 4)
 		name = string_append(name, "Windows NT");
-            else {
+	    else {
 		sprintf(tbuf, "Windows Unknown (%ld.%ld)",
 		    osv.dwMajorVersion, osv.dwMinorVersion); 
 		name = string_append(name, tbuf);
-    	    }
+	    }
 
-            /* service pack and build number etc */
-            if (osv.szCSDVersion[0] != '\0') {
+	    /* service pack and build number etc */
+	    if (osv.szCSDVersion[0] != '\0') {
 		name = string_append(name, " ");
 		name = string_append(name, osv.szCSDVersion);
 	    }
 	    sprintf(tbuf, " Build %ld", osv.dwBuildNumber & 0xFFFF);
-	    name = string_append(name, tbuf);
+	    windows_build = name + strlen(name) + 1;
+	    windows_uname = string_append(name, tbuf);
 	    break;
 
         default:
-	    name = string_append(name, "Windows - Platform Unknown");
+	    windows_uname = "Windows - Platform Unknown";
+	    windows_build = "Unknown Build";
 	    break;
     }
-    return name;
 }
 
 void
-setup_globals(void)
+windows_setup_globals(void)
 {
     SYSTEM_INFO		sysinfo;
     OSVERSIONINFOEX	osversion;
-#if (_WIN32_WINNT >= 0x0500)
-    MEMORYSTATUSEX	memstat;
-#else
-    MEMORYSTATUS	memstat;
-#endif
-
-    /* physical memory size (in units of bytes) */
-#if (_WIN32_WINNT >= 0x0500)
-    ZeroMemory(&memstat, sizeof(MEMORYSTATUSEX));
-    if (!GlobalMemoryStatusEx(&memstat))
-	__pmNotifyErr(LOG_ERR, "GlobalMemoryStatusEx failed");
-    else
-	windows_physmem = memstat.ullTotalPhys / (1024*1024);
-#else
-    ZeroMemory(&memstat, sizeof(MEMORYSTATUS));
-    GlobalMemoryStatus(&memstat);
-    windows_physmem = memstat.dwTotalPhys / (1024*1024);
-#endif
 
     ZeroMemory(&sysinfo, sizeof(SYSTEM_INFO));
     GetSystemInfo(&sysinfo);
     windows_pagesize = sysinfo.dwPageSize;
 
+    switch (sysinfo.wProcessorArchitecture) {
+    case PROCESSOR_ARCHITECTURE_AMD64:
+	windows_machine = "x86_64";
+	break;
+    case PROCESSOR_ARCHITECTURE_IA64:
+	windows_machine = "ia64";
+	break;
+    case PROCESSOR_ARCHITECTURE_INTEL:
+	windows_machine = "i686";
+	break;
+    default:
+	windows_machine = "Unknown";
+	break;
+    }
+
     ZeroMemory(&osversion, sizeof(OSVERSIONINFOEX));
     osversion.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);
-    if (!GetVersionEx((OSVERSIONINFO *)&osversion))
-	__pmNotifyErr(LOG_ERR, "GetVersionEx failed");
-    else
-	format_uname(osversion);
+    GetVersionEx((OSVERSIONINFO *)&osversion);
+    windows_format_uname(osversion);
 }
 
-static int
-check_semantics(pdh_metric_t *mp, pdh_value_t *vp)
+static void
+windows_verify_metric(pdh_metric_t *mp, PDH_COUNTER_INFO_A *infop)
 {
-    int		sts = 0;
     char	*ctr_type;
 
+    mp->ctype = infop->dwType;
+
     switch (mp->ctype) {
 	/*
 	 * Pdh metric sematics ... from WinPerf.h
@@ -299,8 +283,6 @@ check_semantics(pdh_metric_t *mp, pdh_value_t *vp)
 	 *	    RATE	time rate converted
 	 *	    FRACTION	divide value by BASE
 	 *	    BASE	used for FRACTION
-	 *	    ELAPSED	subtract from current time
-	 *			(no support here)
 	 *	    QUEUELEN	magic internal queuelen() routines
 	 *			(you're joking, right?)
 	 *	    HISTOGRAM	counter begins or ends a histo (?)
@@ -317,7 +299,9 @@ check_semantics(pdh_metric_t *mp, pdh_value_t *vp)
 	case PERF_COUNTER_COUNTER:
 	    /* 32-bit PM_SEM_COUNTER */
 	    if (mp->desc.type != PM_TYPE_32 && mp->desc.type != PM_TYPE_U32) {
-		__pmNotifyErr(LOG_ERR, "windows_open: PERF_COUNTER_COUNTER: "
+		if (!(mp->flags & M_AUTO64))
+		    __pmNotifyErr(LOG_ERR,
+			"windows_open: PERF_COUNTER_COUNTER: "
 			"metric %s: rewrite type from %s to PM_TYPE_U32\n",
 			pmIDStr(mp->desc.pmid), _typestr(mp->desc.type));
 		mp->desc.type = PM_TYPE_U32;
@@ -338,10 +322,12 @@ check_semantics(pdh_metric_t *mp, pdh_value_t *vp)
 		ctr_type = "PERF_COUNTER_RAWCOUNT_HEX";
 	    /* 32-bit PM_SEM_INSTANT or PM_SEM_DISCRETE */
 	    if (mp->desc.type != PM_TYPE_32 && mp->desc.type != PM_TYPE_U32) {
-		__pmNotifyErr(LOG_ERR, "windows_open: Warning: %s: metric %s: "
-					"rewrite type from %s to PM_TYPE_U32\n",
-					ctr_type, pmIDStr(mp->desc.pmid),
-					_typestr(mp->desc.type));
+		if (!(mp->flags & M_AUTO64))
+		    __pmNotifyErr(LOG_ERR,
+			"windows_open: Warning: %s: metric %s: "
+			"rewrite type from %s to PM_TYPE_U32\n",
+			ctr_type, pmIDStr(mp->desc.pmid),
+			_typestr(mp->desc.type));
 		mp->desc.type = PM_TYPE_U32;
 	    }
 	    break;
@@ -352,7 +338,9 @@ check_semantics(pdh_metric_t *mp, pdh_value_t *vp)
 	case PERF_COUNTER_BULK_COUNT:
 	    /* 64-bit PM_SEM_COUNTER */
 	    if (mp->desc.type != PM_TYPE_64 && mp->desc.type != PM_TYPE_U64) {
-		__pmNotifyErr(LOG_ERR, "windows_open: PERF_COUNTER_BULK_COUNT:"
+		if (!(mp->flags & M_AUTO64))
+		    __pmNotifyErr(LOG_ERR,
+			"windows_open: PERF_COUNTER_BULK_COUNT:"
 			" metric %s: rewrite type from %s to PM_TYPE_U64\n",
 			pmIDStr(mp->desc.pmid), _typestr(mp->desc.type));
 		mp->desc.type = PM_TYPE_U64;
@@ -377,7 +365,8 @@ check_semantics(pdh_metric_t *mp, pdh_value_t *vp)
 	     * we shall export 'em as microseconds
 	     */
 	    if (mp->desc.type != PM_TYPE_64 && mp->desc.type != PM_TYPE_U64) {
-		__pmNotifyErr(LOG_ERR, "windows_open: Warning: %s: "
+		if (!(mp->flags & M_AUTO64))
+		    __pmNotifyErr(LOG_ERR, "windows_open: Warning: %s: "
 			"metric %s: rewrite type from %s to PM_TYPE_U64\n",
 		    ctr_type, pmIDStr(mp->desc.pmid), _typestr(mp->desc.type));
 		mp->desc.type = PM_TYPE_U64;
@@ -410,7 +399,8 @@ check_semantics(pdh_metric_t *mp, pdh_value_t *vp)
 	    /* 64-bit PM_SEM_INSTANT or PM_SEM_DISCRETE */
 	    if (mp->desc.type != PM_TYPE_64 &&
 		mp->desc.type != PM_TYPE_U64) {
-		__pmNotifyErr(LOG_ERR, "windows_open: Warning: "
+		if (!(mp->flags & M_AUTO64))
+		    __pmNotifyErr(LOG_ERR, "windows_open: Warning: "
 				"PERF_COUNTER_LARGE_RAWCOUNT: metric %s: "
 				"rewrite type from %s to PM_TYPE_U64\n",
 		    pmIDStr(mp->desc.pmid), _typestr(mp->desc.type));
@@ -439,56 +429,191 @@ check_semantics(pdh_metric_t *mp, pdh_value_t *vp)
 		mp->desc.sem = PM_SEM_INSTANT;
 	    }
 	    if (mp->desc.type != PM_TYPE_64 && mp->desc.type != PM_TYPE_U64) {
-		__pmNotifyErr(LOG_ERR, "windows_open: Warning: %s "
+		if (!(mp->flags & M_AUTO64))
+		    __pmNotifyErr(LOG_ERR, "windows_open: Warning: %s "
 			"metric %s: rewrite type from %s to PM_TYPE_U64\n",
 			ctr_type, pmIDStr(mp->desc.pmid), _typestr(mp->desc.type));
 		mp->desc.type = PM_TYPE_U64;
 	    }
 	    break;
 
+	case PERF_SAMPLE_COUNTER:
+	    /* floating point PM_SEM_INSTANT or PM_SEM_DISCRETE */
+	    if (mp->desc.sem != PM_SEM_INSTANT && mp->desc.sem != PM_SEM_DISCRETE) {
+		__pmNotifyErr(LOG_ERR, "windows_open: Warning: %s: "
+				       "metric %s: semantics %s (expected %s)\n",
+					ctr_type, pmIDStr(mp->desc.pmid),
+					_semstr(mp->desc.sem), _semstr(PM_SEM_INSTANT));
+		mp->desc.sem = PM_SEM_INSTANT;
+	    }
+	    if (mp->desc.type != PM_TYPE_FLOAT && mp->desc.type != PM_TYPE_DOUBLE) {
+		__pmNotifyErr(LOG_ERR, "windows_open: Warning: %s "
+			"metric %s: rewrite type from %s to PM_TYPE_FLOAT\n",
+			ctr_type, pmIDStr(mp->desc.pmid), _typestr(mp->desc.type));
+		mp->desc.type = PM_TYPE_FLOAT;
+	    }
+	    break;
+
+	case PERF_ELAPSED_TIME:
+	    if (mp->desc.units.dimSpace != 0 ||
+		mp->desc.units.dimTime != 1 ||
+		mp->desc.units.dimCount != 0 ||
+		mp->desc.units.scaleTime != PM_TIME_USEC) {
+		pmUnits units = mp->desc.units;
+		mp->desc.units.dimSpace = mp->desc.units.dimCount = 0;
+		mp->desc.units.scaleSpace = mp->desc.units.scaleCount = 0;
+		mp->desc.units.dimTime = 1;
+		mp->desc.units.scaleTime = PM_TIME_USEC;
+		__pmNotifyErr(LOG_ERR, "windows_open: Warning: %s: "
+			"metric %s: rewrite dimension and scale from %s to %s",
+		    ctr_type, pmIDStr(mp->desc.pmid), pmUnitsStr(&units),
+		    pmUnitsStr(&mp->desc.units));
+	    }
+	    if (mp->desc.type != PM_TYPE_64 && mp->desc.type != PM_TYPE_U64) {
+		if (!(mp->flags & M_AUTO64))
+		    __pmNotifyErr(LOG_ERR, "windows_open: Warning: %s "
+			"metric %s: rewrite type from %s to PM_TYPE_U64\n",
+			ctr_type, pmIDStr(mp->desc.pmid),
+			_typestr(mp->desc.type));
+		mp->desc.type = PM_TYPE_U64;
+	    }
+
 	default:
-	    sts = -1;
+	    __pmNotifyErr(LOG_ERR, "windows_open: Warning: metric %s: "
+				   "unexpected counter type: %s\n",
+			pmIDStr(mp->desc.pmid), decode_ctype(infop->dwType));
     }
     mp->flags |= M_EXPANDED;
-    return sts;
 }
 
 int
-windows_check_metric(pdh_metric_t *pmp)
+windows_inform_metric(pdh_metric_t *pmp, LPTSTR p, pdh_value_t *pvp,
+			BOOLEAN getExplainText, pdh_metric_inform_t informer)
 {
-    int			i, id;
-    size_t		size;
+    int			sts = -1;
     PDH_STATUS  	pdhsts;
-    static LPSTR	pattern = NULL;
-    static DWORD	pattern_sz = 0;
-    static LPSTR	info = NULL;
+    PDH_HQUERY		queryhdl = NULL;
+    PDH_HCOUNTER	counterhdl = NULL;
+    DWORD		result_sz;
     static DWORD	info_sz = 0;
-    static DWORD	result_sz;
-    PDH_COUNTER_INFO_A	*infop;
-    LPTSTR      	p;
+    static LPSTR	info = NULL;
 
-    for (i = 0; i < pmp->num_vals; i++)
-	PdhRemoveCounter(pmp->vals[i].hdl);
-    if (pmp->num_vals)
-	free(pmp->vals);
-    pmp->num_vals = 0;
-    pmp->flags &= ~(M_EXPANDED|M_NOVALUES);
+    pdhsts = PdhOpenQueryA(NULL, 0, &queryhdl);
+    if (pdhsts != ERROR_SUCCESS) {
+	__pmNotifyErr(LOG_ERR, "windows_open: PdhOpenQueryA failed: %s\n",
+			pdherrstr(pdhsts));
+	return sts;
+    }
 
-    result_sz = pattern_sz;
-    pdhsts = PdhExpandCounterPathA(pmp->pat, pattern, &result_sz);
-    if (pdhsts == PDH_MORE_DATA) {
-	result_sz++;		// not sure if this is necessary?
-	pattern_sz = result_sz;
-	if ((pattern = (LPSTR)realloc(pattern, pattern_sz)) == NULL) {
-	    __pmNotifyErr(LOG_ERR, "windows_open: PdhExpandCounterPathA "
-				   "realloc (%d) failed @ metric %s: ",
-				(int)pattern_sz, pmIDStr(pmp->desc.pmid));
-	    errmsg();
-	    return -1;
+    pdhsts = PdhAddCounterA(queryhdl, p, pvp->inst, &counterhdl);
+    if (pdhsts != ERROR_SUCCESS) {
+	__pmNotifyErr(LOG_ERR, "windows_open: Warning: PdhAddCounterA "
+				"@ pmid=%s pat=\"%s\": %s\n",
+			    pmIDStr(pmp->desc.pmid), p, pdherrstr(pdhsts));
+	PdhCloseQuery(queryhdl);
+	return sts;
+    }
+
+    /*
+     * check PCP metric semantics against PDH info
+     */
+    if (info_sz == 0) {
+	/*
+	 * We've observed an initial call to PdhGetCounterInfoA()
+	 * hang with a zero sized buffer ... pander to this with
+	 * an initial buffer allocation ... (size is a 100% guess).
+	 */
+	info_sz = 256;
+	if ((info = (LPSTR)malloc(info_sz)) == NULL) {
+	    __pmNotifyErr(LOG_ERR, "windows_open: PdhGetCounterInfoA "
+				   "malloc (%d) failed @ metric %s: ",
+				(int)info_sz, pmIDStr(pmp->desc.pmid));
+	    goto done;
 	}
-	pdhsts = PdhExpandCounterPathA(pmp->pat, pattern, &result_sz);
+    }
+    result_sz = info_sz;
+    pdhsts = PdhGetCounterInfoA(counterhdl, getExplainText, &result_sz,
+					(PDH_COUNTER_INFO_A *)info);
+    if (pdhsts == PDH_MORE_DATA) {
+	info_sz = result_sz;
+	if ((info = (LPSTR)realloc(info, info_sz)) == NULL) {
+	    __pmNotifyErr(LOG_ERR, "windows_open: PdhGetCounterInfoA "
+				   "realloc (%d) failed @ metric %s: ",
+				(int)info_sz, pmIDStr(pmp->desc.pmid));
+	    goto done;
+	}
+	pdhsts = PdhGetCounterInfoA(counterhdl, getExplainText, &result_sz,
+				    (PDH_COUNTER_INFO_A *)info);
     }
     if (pdhsts != ERROR_SUCCESS) {
+	__pmNotifyErr(LOG_ERR, "windows_open: PdhGetCounterInfoA "
+				"failed @ metric %s: %s\n",
+				pmIDStr(pmp->desc.pmid), pdherrstr(pdhsts));
+	goto done;
+    }
+    else {
+	informer(pmp, (PDH_COUNTER_INFO_A *)info);
+	sts = 0;
+    }
+
+done:
+    PdhRemoveCounter(counterhdl);
+    PdhCloseQuery(queryhdl);
+    return sts;
+}
+
+void
+windows_verify_callback(pdh_metric_t *pmp, LPSTR pat, pdh_value_t *pvp)
+{
+    int	v;
+
+    if (!(pmp->flags & M_VERIFIED)) {
+	v = windows_inform_metric(pmp, pat, pvp, FALSE, windows_verify_metric);
+	if (v == 0)
+	    pmp->flags |= M_VERIFIED;
+    }
+}
+
+
+/*
+ * General purpose metric regex iterator, call out on each instance
+ */
+int
+windows_visit_metric(pdh_metric_t *pmp, pdh_metric_visitor_t visitor)
+{
+    size_t		size;
+    int			index = 0;
+    PDH_STATUS  	pdhsts;
+    DWORD		result_sz;
+    static DWORD	pattern_sz = 0;
+    static LPSTR	pattern = NULL;
+    LPSTR      		p;
+
+    if (pmp->desc.indom != PM_INDOM_NULL) {
+	index = pmInDom_serial(pmp->desc.indom);
+	pmdaCacheOp(pmp->desc.indom, PMDA_CACHE_INACTIVE);
+    }
+
+    pmp->flags &= ~(M_EXPANDED|M_NOVALUES);
+    memset(pmp->vals, 0, pmp->num_alloc * sizeof(pdh_value_t));
+    pmp->num_vals = 0;
+
+    result_sz = 0;
+    pdhsts = PdhExpandCounterPathA(pmp->pat, NULL, &result_sz);
+    if (pdhsts == PDH_MORE_DATA) {
+	if (result_sz >= pattern_sz) {
+	    pattern_sz = roundup(result_sz, 64);
+	    if ((pattern = (LPSTR)realloc(pattern, pattern_sz)) == NULL) {
+		__pmNotifyErr(LOG_ERR, "windows_open: PdhExpandCounterPathA "
+					"realloc (%ld) failed @ metric %s: ",
+				pattern_sz, pmIDStr(pmp->desc.pmid));
+		return -1;
+	    }
+	}
+	result_sz = pattern_sz;
+	pdhsts = PdhExpandCounterPathA(pmp->pat, pattern, &result_sz);
+    }
+    if (pdhsts != PDH_CSTATUS_VALID_DATA) {
 	if (pmp->pat[0] == '\0') {
 	    /*
 	     * Empty path string.  Used to denote metrics that are
@@ -523,15 +648,19 @@ windows_check_metric(pdh_metric_t *pmp)
 	pdh_value_t	*pvp;
 
 	pmp->num_vals++;
-	size = pmp->num_vals * sizeof(pdh_value_t);
-	if ((pmp->vals = (pdh_value_t *)realloc(pmp->vals, size)) == NULL) {
-	    __pmNotifyErr(LOG_ERR, "windows_open: Error: values realloc "
+	if (pmp->num_vals > pmp->num_alloc) {
+	    size = pmp->num_vals * sizeof(pdh_value_t);
+	    if ((pmp->vals = (pdh_value_t *)realloc(pmp->vals, size)) == NULL) {
+		__pmNotifyErr(LOG_ERR, "windows_open: Error: values realloc "
 				   "(%d x %d) failed @ metric %s [%s]: ",
 				pmp->num_vals, sizeof(pdh_value_t),
 				pmIDStr(pmp->desc.pmid), p);
-	    errmsg();
-	    return -1;
+		pmp->num_alloc = 0;
+		return -1;
+	    }
+	    pmp->num_alloc = pmp->num_vals;
 	}
+
 	pvp = &pmp->vals[pmp->num_vals-1];
 	if (pmp->desc.indom == PM_INDOM_NULL) {
 	    /* singular instance */
@@ -564,7 +693,7 @@ windows_check_metric(pdh_metric_t *pmp)
 	     * indom type to extract instance name and number, and
 	     * add into indom cache data structures as needed.
 	     */
-	    if ((pvp->inst = windows_check_instance(p, pmp)) < 0) {
+	    if ((pvp->inst = windows_lookup_instance(p, pmp)) < 0) {
 		/*
 		 * error reported in windows_check_instance() ...
 		 * we cannot return any values for this instance if
@@ -576,101 +705,24 @@ windows_check_metric(pdh_metric_t *pmp)
 		/* move onto next instance */
 		continue;
 	    }
+	    windows_indom_setup[index] = 1;
 	}
 
-	/*
-	 * Note.  Passing inst down here does not help much, as
-	 * I don't think we're ever going to navigate back from a PDH
-	 * counter into our data structures, and even if we do,
-	 * the instance id is not going to be unique enough.
-	 */
-	id = pmp->qid;
-	pdhsts = PdhAddCounterA(querydesc[id].hdl, p, pvp->inst, &pvp->hdl);
-	if (pdhsts != ERROR_SUCCESS) {
-	    __pmNotifyErr(LOG_ERR, "windows_open: Warning: PdhAddCounterA "
-				"@ pmid=%s inst=%d pat=\"%s\" hdl=%p: %s\n",
-			    pmIDStr(pmp->desc.pmid), pvp->inst, p, pvp->hdl,
-			    pdherrstr(pdhsts));
-	    pmp->flags |= M_NOVALUES;
-	    break;
-	}
-
-	if (p > pattern)
-	    continue;
-	if (pvp->flags & V_VERIFIED)
-	    continue;
-
-	/*
-	 * check PCP metric semantics against PDH info
-	 */
-	if (info_sz == 0) {
-	    /*
-	     * We've observed an initial call to PdhGetCounterInfoA()
-	     * hang with a zero sized buffer ... pander to this with
-	     * an initial buffer allocation ... (size is a 100% guess).
-	     */
-	    info_sz = 256;
-	    if ((info = (LPSTR)malloc(info_sz)) == NULL) {
-		__pmNotifyErr(LOG_ERR, "windows_open: PdhGetCounterInfoA "
-					"malloc (%d) failed @ metric %s: ",
-				(int)info_sz, pmIDStr(pmp->desc.pmid));
-		errmsg();
-		return -1;
-	    }
-	}
-	result_sz = info_sz;
-	pdhsts = PdhGetCounterInfoA(pvp->hdl, FALSE, &result_sz,
-					(PDH_COUNTER_INFO_A *)info);
-	if (pdhsts == PDH_MORE_DATA) {
-	    info_sz = result_sz;
-	    if ((info = (LPSTR)realloc(info, info_sz)) == NULL) {
-		__pmNotifyErr(LOG_ERR, "windows_open: PdhGetCounterInfoA "
-					"realloc (%d) failed @ metric %s: ",
-				(int)info_sz, pmIDStr(pmp->desc.pmid));
-		errmsg();
-		return -1;
-	    }
-	    pdhsts = PdhGetCounterInfoA(pvp->hdl, FALSE, &result_sz,
-					(PDH_COUNTER_INFO_A *)info);
-	}
-	if (pdhsts != ERROR_SUCCESS) {
-	    __pmNotifyErr(LOG_ERR, "windows_open: PdhGetCounterInfoA "
-					"failed @ metric %s: %s\n",
-				pmIDStr(pmp->desc.pmid), pdherrstr(pdhsts));
-	    continue;
-	}
-	infop = (PDH_COUNTER_INFO_A *)info;
-	pmp->ctype = infop->dwType;
-
-	if (check_semantics(pmp, pvp) == 0)
-	    pvp->flags |= V_VERIFIED;
-	else
-	    __pmNotifyErr(LOG_ERR, "windows_open: Warning: metric %s: "
-				   "unexpected counter type: %s\n",
-			pmIDStr(pmp->desc.pmid), decode_ctype(pmp->ctype));
+	if (visitor)
+	    visitor(pmp, p, pvp);
     }
+
     return 0;
 }
 
 void
 windows_open(void)
 {
-    int			i, sts = 0;
+    windows_setup_globals();
 
-    setup_globals();
-
-    memset(querydesc, 0, sizeof(pdh_query_t) * Q_NUMQUERIES);
-    for (i = 0; i < Q_NUMQUERIES; i++) {
-	sts = PdhOpenQueryA(NULL, 0, &querydesc[i].hdl);
-	if (sts != ERROR_SUCCESS) {
-	    __pmNotifyErr(LOG_ERR, "windows_open: PdhOpenQueryA "
-				    "failed @ query %d: %s\n",
-				    i, pdherrstr(sts));
-	    querydesc[i].flags |= Q_ERR_SEEN;
-	}
-    }
-
-    for (i = 0; i < metricdesc_sz; i++) {
-	windows_check_metric(&metricdesc[i]);
+    if (pmDebug & DBG_TRACE_LIBPMDA) {
+	int i;
+	for (i = 0; i < metricdesc_sz; i++)
+	    windows_visit_metric(&metricdesc[i], windows_verify_callback);
     }
 }
