@@ -14,13 +14,8 @@
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
- * 
- * Contact information: Silicon Graphics, Inc., 1500 Crittenden Lane,
- * Mountain View, CA 94043, USA, or: http://www.sgi.com
  */
 
-#ident "$Id: pmdbg.c,v 1.2 2003/02/20 19:20:25 kenmcd Exp $"
-
 /*
  * pmdbg - help for PCP debug flags
  *
@@ -86,17 +81,9 @@ main(int argc, char **argv)
     int		i;
     int		c;
     int		code;
-    char	*p;
     int		errflag = 0;
-    extern char	*optarg;
-    extern int	optind;
 
-    /* trim command name of leading directory components */
-    pmProgname = argv[0];
-    for (p = pmProgname; *p; p++) {
-	if (*p == '/')
-	    pmProgname = p+1;
-    }
+    __pmSetProgname(argv[0]);
 
     while ((c = getopt(argc, argv, "l?")) != EOF) {
 	switch (c) {
@@ -107,7 +94,6 @@ main(int argc, char **argv)
 	    for (i = 0; i < nfoo; i++)
 		printf(fmt, foo[i].name, foo[i].flag, foo[i].text);
 	    exit(0);
-	    /*NOTREACHED*/
 
 	case '?':
 	default:
