@@ -9,8 +9,6 @@
 #	on-off	y or n to enable or disable this group
 #	delta	delta argument for pmlogger "logging ... on delta" clause
 #	
-# $Id: pmlogconf.sh,v 1.4 2003/06/19 07:32:45 kenmcd Exp $
-#
 #
 # Copyright (c) 1998,2003 Silicon Graphics, Inc.  All Rights Reserved.
 # 
@@ -27,12 +25,9 @@
 # You should have received a copy of the GNU General Public License along
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
-# 
-# Contact information: Silicon Graphics, Inc., 1500 Crittenden Lane,
-# Mountain View, CA 94043, USA, or: http://www.sgi.com
 
 # Get standard environment
-. /etc/pcp.env
+. $PCP_DIR/etc/pcp.env
 
 prog=`basename $0`
 
@@ -57,7 +52,6 @@ do
 
 	?)	# eh?
 		_usage
-		# NOTREACHED
 		;;
     esac
 done
@@ -66,7 +60,6 @@ shift `expr $OPTIND - 1`
 if [ $# -ne 1 ]
 then
     _usage
-    # NOTREACHED
 fi
 
 config=$1
@@ -166,7 +159,7 @@ BEGIN						{ out = "'"$tmp.pre"'" }
 		    metrics="hinv"
 		    ;;
 
-	    D0)	desc="activity (IOPs and bytes for both reads and writes) over all disks [osvis, pmkstat, pmchart:Disk, pmchart:Overview]"
+	    D0)	desc="activity (IOPs and bytes for both reads and writes) over all disks [osvis, pmstat, pmchart:Disk, pmchart:Overview]"
 		    metrics="disk.all.read
 			    disk.all.write
 			    disk.all.total
@@ -199,7 +192,7 @@ BEGIN						{ out = "'"$tmp.pre"'" }
 		    metrics="disk.dev"
 		    ;;
 
-	    C0)	desc="utilization (usr, sys, idle, ...) over all CPUs [osvis, pmkstat, pmchart:CPU, pmchart:Overview, pmie:cpu]"
+	    C0)	desc="utilization (usr, sys, idle, ...) over all CPUs [osvis, pmstat, pmchart:CPU, pmchart:Overview, pmie:cpu]"
 		    metrics="kernel.all.cpu.idle
 			    kernel.all.cpu.nice
 			    kernel.all.cpu.intr
@@ -227,13 +220,13 @@ BEGIN						{ out = "'"$tmp.pre"'" }
 	            metrics="kernel.percpu.wait"
 		    ;;
 
-	    K0)	desc="load average and number of logins [osvis, pmkstat, pmchart:LoadAvg, pmchart:Overview, pmie:cpu]"
+	    K0)	desc="load average and number of logins [osvis, pmstat, pmchart:LoadAvg, pmchart:Overview, pmie:cpu]"
 		    metrics="kernel.all.load
 			    kernel.all.users"
 
 		    ;;
 
-	    K1)	desc="context switches, total syscalls and counts for selected calls (e.g. read, write, fork, exec, select) over all CPUs [pmkstat, pmchart:Syscalls, pmie:cpu]"
+	    K1)	desc="context switches, total syscalls and counts for selected calls (e.g. read, write, fork, exec, select) over all CPUs [pmstat, pmchart:Syscalls, pmie:cpu]"
 		    metrics="kernel.all.pswitch
 			    kernel.all.syscall
 			    kernel.all.sysexec
@@ -305,7 +298,7 @@ BEGIN						{ out = "'"$tmp.pre"'" }
 			    kernel.all.swap.swpocc"
 		    ;;
 
-	    M0)	desc="pages in and out (severe VM demand) [pmkstat, pmchart:Paging]"
+	    M0)	desc="pages in and out (severe VM demand) [pmstat, pmchart:Paging]"
 		    metrics="swap.pagesin
 			    swap.pagesout"
 		    ;;
@@ -314,7 +307,7 @@ BEGIN						{ out = "'"$tmp.pre"'" }
 		    metrics="mem.fault mem.tlb"
 		    ;;
 
-	    M2)	desc="kernel memory allocation [osvis, pmkstat, pmchart:Memory, pmchart:Overview]"
+	    M2)	desc="kernel memory allocation [osvis, pmstat, pmchart:Memory, pmchart:Overview]"
 		    metrics="mem.system
 			    mem.util
 			    mem.freemem
