@@ -14,31 +14,12 @@
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
- * 
- * Contact information: Silicon Graphics, Inc., 1500 Crittenden Lane,
- * Mountain View, CA 94043, USA, or: http://www.sgi.com
  */
 
-#ident "$Id: dopdu.c,v 1.6 2006/05/02 05:57:28 makc Exp $"
-
-#include <stdio.h>
-#include <unistd.h>
-#include <errno.h>
-#include <syslog.h>
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <netdb.h>
-#include <time.h>
 #include "pmapi.h"
 #include "impl.h"
-#include "./logger.h"
+#include "logger.h"
 
-#if defined(IRIX6_5)
-#include <optional_sym.h>
-#endif
-
-extern int		pmDebug;
 extern int		clientfd;
 
 
@@ -287,7 +268,6 @@ add_metric(pmValueSet *vsp, task_t **result)
     dp = (pmDesc *)malloc(sizeof(pmDesc));
     if (dp == NULL) {
 	__pmNoMem("add_metric: new pmDesc malloc", sizeof(pmDesc), PM_FATAL_ERR);
-	/*NOTREACHED*/
     }
     if ((sts = pmLookupDesc(pmid, dp)) < 0)
 	return sts;
@@ -297,7 +277,6 @@ add_metric(pmValueSet *vsp, task_t **result)
 	tp = calloc(1, sizeof(task_t));
 	if (tp == NULL) {
 	    __pmNoMem("add_metric: new task calloc", sizeof(task_t), PM_FATAL_ERR);
-	    /*NOTREACHED*/
 	}
 	*result = tp;
     }
@@ -308,19 +287,16 @@ add_metric(pmValueSet *vsp, task_t **result)
     tp->t_pmidlist = (pmID *)realloc(tp->t_pmidlist, need);
     if (tp->t_pmidlist == NULL) {
 	__pmNoMem("add_metric: new task pmidlist realloc", need, PM_FATAL_ERR);
-	/*NOTREACHED*/
     }
     need = tp->t_numpmid * sizeof(char *);
     tp->t_namelist = (char **)realloc(tp->t_namelist, need);
     if (tp->t_namelist == NULL) {
 	__pmNoMem("add_metric: new task namelist realloc", need, PM_FATAL_ERR);
-	/*NOTREACHED*/
     }
     need = tp->t_numpmid * sizeof(pmDesc);
     tp->t_desclist = (pmDesc *)realloc(tp->t_desclist, need);
     if (tp->t_desclist == NULL) {
 	__pmNoMem("add_metric: new task desclist realloc", need, PM_FATAL_ERR);
-	/*NOTREACHED*/
     }
     tp->t_namelist[tp->t_numpmid-1] = NULL;	/* name not known here */
     tp->t_pmidlist[tp->t_numpmid-1] = pmid;
@@ -329,7 +305,6 @@ add_metric(pmValueSet *vsp, task_t **result)
     rqp = (optreq_t *)calloc(1, sizeof(optreq_t));
     if (rqp == NULL) {
 	__pmNoMem("add_metric: new task optreq calloc", need, PM_FATAL_ERR);
-	/*NOTREACHED*/
     }
     rqp->r_desc = dp;
 
@@ -348,7 +323,6 @@ add_metric(pmValueSet *vsp, task_t **result)
 	if (rqp->r_instlist == NULL) {
 	    __pmNoMem("add_metric: new task optreq instlist malloc", need,
 		     PM_FATAL_ERR);
-	    /*NOTREACHED*/
 	}
 	for (i = 0; i < vsp->numval; i++)
 	    rqp->r_instlist[i] = vsp->vlist[i].inst;
@@ -510,7 +484,6 @@ update_metric(pmValueSet *vsp, int reqstate, int mflags, task_t **result)
 	if (ntp == NULL) {
 	    __pmNoMem("update_metric: new task calloc", sizeof(task_t),
 		     PM_FATAL_ERR);
-	    /*NOTREACHED*/
 	}
 	*result = ntp;
     }
@@ -624,7 +597,6 @@ update_metric(pmValueSet *vsp, int reqstate, int mflags, task_t **result)
 		    if (rqp == NULL) {
 			__pmNoMem("update_metric: optreq calloc",
 				 sizeof(optreq_t), PM_FATAL_ERR);
-			/*NOTREACHED*/
 		    }
 		    /* if the metric existed but the instance didn't, we don't
 		     * have a valid pmDesc (dp), so find one.
@@ -651,7 +623,6 @@ update_metric(pmValueSet *vsp, int reqstate, int mflags, task_t **result)
 			if (rqp->r_desc == NULL) {
 			    __pmNoMem("update_metric: new inst pmDesc malloc",
 				     need, PM_FATAL_ERR);
-			    /*NOTREACHED*/
 			}
 			memcpy(rqp->r_desc, dp, need);
 		    }
@@ -664,7 +635,6 @@ update_metric(pmValueSet *vsp, int reqstate, int mflags, task_t **result)
 		if (rqp->r_instlist == NULL) {
 		    __pmNoMem("update_metric: inst list resize", need,
 			     PM_FATAL_ERR);
-		    /*NOTREACHED*/
 		}
 		rqp->r_instlist[rqp->r_numinst++] = inst;
 		if ((sts = __pmOptFetchAdd(&ntp->t_fetch, rqp)) < 0)
@@ -743,7 +713,6 @@ update_metric(pmValueSet *vsp, int reqstate, int mflags, task_t **result)
 		if (rqp == NULL) {
 		    __pmNoMem("update_metric: all inst calloc",
 			     sizeof(optreq_t), PM_FATAL_ERR);
-		    /*NOTREACHED*/
 		}
 		rqp->r_desc = dp;
 		if ((sts = __pmOptFetchAdd(&ntp->t_fetch, rqp)) < 0)
@@ -769,7 +738,6 @@ update_metric(pmValueSet *vsp, int reqstate, int mflags, task_t **result)
 	ntp->t_pmidlist = (pmID *)realloc(ntp->t_pmidlist, need);
 	if (ntp->t_pmidlist == NULL) {
 	    __pmNoMem("update_metric: grow task pmidlist", need, PM_FATAL_ERR);
-	    /*NOTREACHED*/
 	}
 	ntp->t_pmidlist[ntp->t_numpmid] = pmid;
 	ntp->t_numpmid++;
@@ -858,7 +826,6 @@ siamise_request(pmResult *request)
     result = (pmResult *)malloc(need);
     if (result == NULL) {
 	__pmNoMem("siamise_request: malloc pmResult", need, PM_FATAL_ERR);
-	/*NOTREACHED*/
     }
     for (i = 0; i < request->numpmid; i++) {
 	vsp = request->vset[i];
@@ -955,7 +922,6 @@ no_info:
     vsp = (pmValueSet *)malloc(need);
     if (vsp == NULL) {
 	__pmNoMem("build_vset for control/enquire", need, PM_FATAL_ERR);
-	/*NOTREACHED*/
     }
     vsp->pmid = pmid;
     if (have_desc < 0) {
@@ -1278,7 +1244,6 @@ do_control(__pmPDU *pb)
 	free(result);
     }
     pmFreeResult(request);
-    return;
 }
 
 /*
@@ -1290,7 +1255,7 @@ sendstatus(void)
 {
     int				rv;
     int				end;
-    __pmIPC			*ipc;
+    int				version;
     static int			firsttime = 1;
     static char			*tzlogger;
     extern struct timeval	last_stamp;
@@ -1298,21 +1263,14 @@ sendstatus(void)
     struct hostent		*hep = NULL;
 
     if (firsttime) {
-#if defined(IRIX6_5)
-        if (_MIPS_SYMBOL_PRESENT(__pmTimezone))
-            tzlogger = __pmTimezone();
-        else
-            tzlogger = getenv("TZ");
-#else
         tzlogger = __pmTimezone();
-#endif
 	firsttime = 0;
     }
 
-    if ((end = __pmFdLookupIPC(clientfd, &ipc)) < 0)
-	return end;
+    if ((version = __pmVersionIPC(clientfd)) < 0)
+	return version;
 
-    if (ipc->version >= LOG_PDU_VERSION2) {
+    if (version >= LOG_PDU_VERSION2) {
 	__pmLoggerStatus		ls;
 
 	if ((ls.ls_state = logctl.l_state) == PM_LOG_STATE_NEW)
@@ -1350,89 +1308,13 @@ sendstatus(void)
 	    end = 0;
 	ls.ls_tzlogger[end] = '\0';
 
-	rv =  __pmSendLogStatus(clientfd, &ls);
+	rv = __pmSendLogStatus(clientfd, &ls);
     }
-#ifdef HAVE_V1_SUPPORT
-    else {
-	__pmLoggerStatus_v1		ls_v1;
-
-	if ((ls_v1.ls_state = logctl.l_state) == PM_LOG_STATE_NEW)
-	    ls_v1.ls_start.tv_sec = ls_v1.ls_start.tv_usec = 0;
-	else
-	    memcpy(&ls_v1.ls_start, &logctl.l_label.ill_start, sizeof(ls_v1.ls_start));
-	memcpy(&ls_v1.ls_last, &last_stamp, sizeof(ls_v1.ls_last));
-	gettimeofday(&now, NULL);
-	ls_v1.ls_timenow.tv_sec = (__int32_t)now.tv_sec;
-	ls_v1.ls_timenow.tv_usec = (__int32_t)now.tv_usec;
-	ls_v1.ls_vol = logctl.l_curvol;
-	ls_v1.ls_size = ftell(logctl.l_mfp);
-
-	/* be careful of buffer size mismatches when copying strings */
-	end = sizeof(ls_v1.ls_hostname) - 1;
-	strncpy(ls_v1.ls_hostname, logctl.l_label.ill_hostname, end);
-	ls_v1.ls_hostname[end] = '\0';
-
-	end = sizeof(ls_v1.ls_tz) - 1;
-	strncpy(ls_v1.ls_tz, logctl.l_label.ill_tz, end);
-	ls_v1.ls_tz[end] = '\0';
-	end = sizeof(ls_v1.ls_tzlogger) - 1;
-	if (tzlogger != NULL)
-	    strncpy(ls_v1.ls_tzlogger, tzlogger, end);
-	else
-	    end = 0;
-	ls_v1.ls_tzlogger[end] = '\0';
-
-	rv = __pmSendDataX(clientfd, PDU_BINARY, PMLC_PDU_STATUS, sizeof(ls_v1), &ls_v1);
-    }
-#endif
-    return (rv);
+    else
+	rv = PM_ERR_IPC;
+    return rv;
 }
 
-#ifdef HAVE_V1_SUPPORT
-/*
- * do_data_x
- * (function kept for backwards compatability with PCP 1.x)
- */
-static int
-do_data_x(__pmPDU *pb)
-{
-    int			sts;
-    int			subtype, vlen;
-    void		*vp;
-
-    if ((sts = __pmDecodeDataX(pb, PDU_BINARY, &subtype, &vlen, &vp)) < 0) {
-	__pmNotifyErr(LOG_ERR,
-		     "do_data_x: error decoding PDU: %s\n", pmErrStr(sts));
-	sts = PM_ERR_IPC;
-    }
-    else {
-	switch (subtype) {
-	    case PMLC_PDU_STATUS_REQ:
-		sts = sendstatus();
-		break;
-
-	    case PMLC_PDU_NEWVOLUME:
-		sts = newvolume(VOL_SW_PMLC);
-		if (sts >= 0)
-		    sts = logctl.l_label.ill_vol;
-		__pmSendError(clientfd, PDU_BINARY, sts);
-		break;
-
-	    case PMLC_PDU_SYNC:
-		sts = do_flush();
-		__pmSendError(clientfd, PDU_BINARY, sts);
-		break;
-
-	    default:
-		sts = PM_ERR_IPC;
-		break;
-	}
-	__pmUnpinPDUBuf(pb);
-    }
-    return sts;
-}
-#endif
-
 static int
 do_request(__pmPDU *pb)
 {
@@ -1473,10 +1355,10 @@ do_creds(__pmPDU *pb)
 {
     int		i;
     int		sts;
+    int		version = UNKNOWN_VERSION;
     int		credcount;
     int		sender;
     __pmCred	*credlist = NULL;
-    __pmIPC	ipc = { UNKNOWN_VERSION, NULL };
 
     if ((sts = __pmDecodeCreds(pb, PDU_BINARY, &sender, &credcount, &credlist)) < 0) {
 	__pmNotifyErr(LOG_ERR, "do_creds: error decoding PDU: %s\n", pmErrStr(sts));
@@ -1485,8 +1367,8 @@ do_creds(__pmPDU *pb)
 
     for (i = 0; i < credcount; i++) {
 	if (credlist[i].c_type == CVERSION) {
-	    ipc.version = credlist[i].c_vala;
-	    if ((sts = __pmAddIPC(clientfd, ipc)) < 0) {
+	    version = credlist[i].c_vala;
+	    if ((sts = __pmSetVersionIPC(clientfd, version)) < 0) {
 		free(credlist);
 		return sts;
 	    }
@@ -1498,7 +1380,7 @@ do_creds(__pmPDU *pb)
 
 #ifdef PCP_DEBUG
     if (pmDebug & DBG_TRACE_APPL1)
-	fprintf(stderr, "do_creds: pmlc version=%d\n", ipc.version);
+	fprintf(stderr, "do_creds: pmlc version=%d\n", version);
 #endif
 
     return sts;
@@ -1513,7 +1395,6 @@ client_req(void)
 {
     int		sts;
     __pmPDU	*pb;
-    __pmIPC	*ipc;
     __pmPDUHdr	*php;
 
     if ((sts = __pmGetPDU(clientfd, PDU_BINARY, TIMEOUT_DEFAULT, &pb)) <= 0) {
@@ -1525,13 +1406,6 @@ client_req(void)
 	return 1;
     }
     php = (__pmPDUHdr *)pb;
-    if ((sts = __pmFdLookupIPC(clientfd, &ipc)) < 0) {
-	fprintf(stderr, "client_req: %s\n", pmErrStr(sts));
-	__pmResetIPC(clientfd);
-	close(clientfd);
-	clientfd = -1;
-	return 1;
-    }
     sts = 0;
 
     switch (php->type) {
@@ -1544,17 +1418,6 @@ client_req(void)
 	case PDU_LOG_CONTROL:	/* version 2 PDU */
 	    do_control(pb);
 	    break;
-#ifdef HAVE_V1_SUPPORT
-	case PDU_DATA_X:	/* version 1 PDU */
-	    ipc->version = LOG_PDU_VERSION1;
-	    sts = do_data_x(pb);
-	    break;
-	case PDU_CONTROL_REQ:	/* version 1 PDU */
-	    if (ipc->version == UNKNOWN_VERSION)
-		ipc->version = LOG_PDU_VERSION1;
-	    do_control(pb);
-	    break;
-#endif
 	default:		/*  unknown PDU  */
 	    fprintf(stderr, "client_req: bad PDU type 0x%x\n", php->type);
 	    sts = PM_ERR_IPC;
@@ -1566,8 +1429,7 @@ client_req(void)
     else {
 	/* the client isn't playing by the rules; disconnect it */
 	__pmSendError(clientfd, PDU_BINARY, sts);
-	__pmResetIPC(clientfd);
-	close(clientfd);
+	__pmCloseSocket(clientfd);
 	clientfd = -1;
 	return 1;
     }
