@@ -299,14 +299,17 @@ add_msg(char **bp, int nchar, char *p)
 void
 do_dialog(char cmd)
 {
-    FILE	*msgf;
+    FILE	*msgf = NULL;
     time_t	now;
-    char	*msg;
     static char	lbuf[100+MAXPATHLEN];
     double	archsize;
     char	*q;
     char	*p = NULL;
     int		nchar;
+    char	*msg;
+#if HAVE_MKSTEMP
+    char	tmp[MAXPATHLEN];
+#endif
 
     /*
      * flush archive buffers so size is accurate
@@ -324,80 +327,88 @@ do_dialog(char cmd)
     nchar = add_msg(&p, 0, "");
     p[0] = '\0';
 
-    sprintf(lbuf, "PCP recording for the archive folio \"%s\" and the host\n", folio_name);
+    sprintf(lbuf, "PCP recording for the archive folio \"%s\" and the host", folio_name);
     nchar = add_msg(&p, nchar, lbuf);
-    sprintf(lbuf, "\"%s\" has been in progress for %ld %s\n",
+    sprintf(lbuf, " \"%s\" has been in progress for %ld %s",
 	pmcd_host,
 	now < 240 ? now : now/60, now < 240 ? "seconds" : "minutes");
     nchar = add_msg(&p, nchar, lbuf);
-    nchar = add_msg(&p, nchar, "and in that time the pmlogger process has created an\n");
-    nchar = add_msg(&p, nchar, "archive of ");
+    nchar = add_msg(&p, nchar, " and in that time the pmlogger process has created an");
+    nchar = add_msg(&p, nchar, " archive of ");
     q = do_size(archsize);
     nchar = add_msg(&p, nchar, q);
-    nchar = add_msg(&p, nchar, ".\n");
+    nchar = add_msg(&p, nchar, ".");
     if (rsc_replay) {
-	nchar = add_msg(&p, nchar, "\nThis archive may be replayed with the following command:\n");
-	sprintf(lbuf, "        $ pmafm %s replay\n", folio_name);
+	nchar = add_msg(&p, nchar, "\n\nThis archive may be replayed with the following command:\n");
+	sprintf(lbuf, "  $ pmafm %s replay", folio_name);
 	nchar = add_msg(&p, nchar, lbuf);
     }
 
     if (cmd == 'D') {
-	nchar = add_msg(&p, nchar, "\nThe application that launched pmlogger has asked pmlogger\n");
-	nchar = add_msg(&p, nchar, "to continue independently and the PCP archive will grow at\n");
-	nchar = add_msg(&p, nchar, "the rate of ");
+	nchar = add_msg(&p, nchar, "\n\nThe application that launched pmlogger has asked pmlogger");
+	nchar = add_msg(&p, nchar, " to continue independently and the PCP archive will grow at");
+	nchar = add_msg(&p, nchar, " the rate of ");
 	q = do_size((archsize * 3600) / now);
 	nchar = add_msg(&p, nchar, q);
 	nchar = add_msg(&p, nchar, " per hour or ");
 	q = do_size((archsize * 3600 * 24) / now);
 	nchar = add_msg(&p, nchar, q);
-	nchar = add_msg(&p, nchar, " per day.\n");
+	nchar = add_msg(&p, nchar, " per day.");
     }
 
     if (cmd == 'X') {
-	nchar = add_msg(&p, nchar, "\nThe application that launched pmlogger has exited and you\n");
-	nchar = add_msg(&p, nchar, "must decide if the PCP recording session should be terminated\n");
-	nchar = add_msg(&p, nchar, "or continued.  If recording is continued the PCP archive will\n");
-	nchar = add_msg(&p, nchar, "grow at the rate of ");
+	nchar = add_msg(&p, nchar, "\n\nThe application that launched pmlogger has exited and you");
+	nchar = add_msg(&p, nchar, " must decide if the PCP recording session should be terminated");
+	nchar = add_msg(&p, nchar, " or continued.  If recording is continued the PCP archive will");
+	nchar = add_msg(&p, nchar, " grow at the rate of ");
 	q = do_size((archsize * 3600) / now);
 	nchar = add_msg(&p, nchar, q);
 	nchar = add_msg(&p, nchar, " per hour or ");
 	q = do_size((archsize * 3600 * 24) / now);
 	nchar = add_msg(&p, nchar, q);
-	nchar = add_msg(&p, nchar, " per day.\n");
+	nchar = add_msg(&p, nchar, " per day.");
     }
 
     if (cmd == 'Q') {
-	nchar = add_msg(&p, nchar, "\nThe application that launched pmlogger has terminated\n");
-	nchar = add_msg(&p, nchar, "this PCP recording session.\n");
+	nchar = add_msg(&p, nchar, "\n\nThe application that launched pmlogger has terminated");
+	nchar = add_msg(&p, nchar, " this PCP recording session.\n");
     }
 
     if (cmd != 'Q') {
-	nchar = add_msg(&p, nchar, "\nAt any time this pmlogger process may be terminated with the\n");
-	nchar = add_msg(&p, nchar, "following command:\n");
-	nchar = add_msg(&p, nchar, "        $ kill -INT");
+	nchar = add_msg(&p, nchar, "\n\nAt any time this pmlogger process may be terminated with the");
+	nchar = add_msg(&p, nchar, " following command:\n");
+	nchar = add_msg(&p, nchar, "  $ kill -s TERM");
 	sprintf(lbuf, " %d\n", (int)getpid());
 	nchar = add_msg(&p, nchar, lbuf);
     }
 
     if (cmd == 'X')
-	nchar = add_msg(&p, nchar, "\nTerminate this PCP recording session now?\n");
+	nchar = add_msg(&p, nchar, "\n\nTerminate this PCP recording session now?");
 
     if (nchar > 0) {
-	char * xconfirm = pmGetConfig("PCP_XCONFIRM_PROG");
+	char * xconfirm = __pmNativePath(pmGetConfig("PCP_XCONFIRM_PROG"));
 	int fd = -1;
 
-	if ((msg = tmpnam(NULL)) == NULL ||
-	    (fd = open(msg, O_WRONLY|O_CREAT|O_EXCL, 0600)) < 0 ||
-	    (msgf = fdopen(fd, "w")) == NULL) {
+#if HAVE_MKSTEMP
+	sprintf(tmp, "%s%cmsgXXXXXX", pmGetConfig("PCP_TMP_DIR"), __pmPathSeparator());
+	msg = tmp;
+	fd = mkstemp(tmp);
+#else
+	if ((msg = tmpnam(NULL)) != NULL)
+	    fd = open(msg, O_WRONLY|O_CREAT|O_EXCL, 0600);
+#endif
+	if (fd >= 0)
+	    msgf = fdopen(fd, "w");
+	if (msgf == NULL) {
 	    fprintf(stderr, "\nError: failed create temporary message file for recording session dialog\n");
 	    fprintf(stderr, "Reason? %s\n", strerror(errno));
 	    if (fd != -1)
 		close(fd);
 	    goto failed;
 	}
-
 	fputs(p, msgf);
 	fclose(msgf);
+	msgf = NULL;
 
 	if (cmd == 'X')
 	    sprintf(lbuf, "%s -c -header \"%s - %s\" -file %s -icon question "
@@ -410,12 +421,12 @@ do_dialog(char cmd)
 
 	if ((msgf = popen(lbuf, "r")) == NULL) {
 	    fprintf(stderr, "\nError: failed to start command for recording session dialog\n");
-	    fprintf(stderr, "Command: \"%s\"\n", msg);
+	    fprintf(stderr, "Command: \"%s\"\n", lbuf);
 	    goto failed;
 	}
 
 	if (fgets(lbuf, sizeof(lbuf), msgf) == NULL) {
-	    fprintf(stderr, "\n%s: xconfirm(1) failed for recording session dialog\n",
+	    fprintf(stderr, "\n%s: pmconfirm(1) failed for recording session dialog\n",
 		    cmd == '?' ? "Warning" : "Error");
 failed:
 	    fprintf(stderr, "Dialog:\n");
@@ -429,7 +440,8 @@ failed:
 	    *q = '\0';
 	}
 
-	pclose(msgf);
+	if (msgf != NULL)
+	    pclose(msgf);
 	unlink(msg);
     }
     else {
@@ -458,6 +470,7 @@ main(int argc, char **argv)
 {
     int			c;
     int			sts;
+    int			sep = __pmPathSeparator();
     int			errflag = 0;
     char		local[MAXHOSTNAMELEN];
     char		*pmnsfile = PM_NS_DEFAULT;
@@ -493,7 +506,9 @@ main(int argc, char **argv)
 		if ( (configfile = (char *)malloc(sz)) == NULL ) {
 		    __pmNoMem("config file name", sz, PM_FATAL_ERR);
 		}
-		sprintf(configfile, "%s/config/pmlogger/%s", vardir, optarg);
+		sprintf(configfile,
+			"%s%c" "config" "%c" "pmlogger" "%c%s",
+			vardir, sep, sep, sep, optarg);
 		if (access(configfile, F_OK) != 0) {
 		    /* still no good, error handling happens below */
 		    free(configfile);
@@ -667,6 +682,26 @@ Options:\n\
 	exit(1);
     }
 
+    if (rsc_fd == -1) {
+	/* no -x, so register client id with pmcd */
+	int	need = 0;
+	int	a;
+	char	*ip;
+	for (a = 0; a < argc; a++)
+	    need += strlen(argv[a])+1;
+	ip = (char *)malloc(need);
+	if (ip != NULL) {
+	    ip[0] = '\0';
+	    for (a = 0; a < argc; a++) {
+		strcat(ip, argv[a]);
+		if (a < argc-1)
+		    strcat(ip, " ");
+	    }
+	    __pmSetClientId(ip);
+	    free(ip);
+	}
+    }
+
     /*
      * discover fd for comms channel to PMCD ... 
      */
@@ -804,8 +839,10 @@ Options:\n\
 
     fprintf(stderr, "Archive basename: %s\n", archBase);
 
+#ifndef IS_MINGW
     /* detach yourself from the launching process */
     setpgid(getpid(), 0);
+#endif
 
     /* set up control port */
     init_ports();
