@@ -14,9 +14,6 @@
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
- * 
- * Contact information: Silicon Graphics, Inc., 1500 Crittenden Lane,
- * Mountain View, CA 94043, USA, or: http://www.sgi.com
  */
 
 #include <math.h>
@@ -414,7 +411,6 @@ main(int argc, char *argv[])
     char		*startstr = NULL;
     char		*endstr = NULL;
     char		*msg = NULL;
-    char		*p;
     int			errflag = 0;
     int			lflag = 0;		/* no label by default */
     pmResult		*result;
@@ -425,14 +421,8 @@ main(int argc, char *argv[])
     struct timeval	delta_stamp;
     int			zflag = 0;		/* for -z */
     char 		*tz = NULL;		/* for -Z timezone */
-    extern char		*optarg;
-    extern int		optind;
-    extern int		pmDebug;
-
-    for (p = pmProgname = argv[0]; *p; p++)
-	if (*p == '/')
-	    pmProgname = p+1;
 
+    __pmSetProgname(argv[0]);
 
     while ((c = getopt(argc, argv, "D:ln:S:T:zZ:?")) != EOF) {
 	switch (c) {
