@@ -39,10 +39,11 @@ usage(void)
 "  -m            also print minimum values\n"
 "  -M            also print maximum values\n"
 "  -n pmnsfile   use an alternative PMNS\n"
-"  -N            suppress warnings from individual archive fetches\n"
+"  -N            suppress warnings from individual archive fetches [default]\n"
 "  -p precision  number of digits to display after the decimal point\n"
 "  -S starttime  start of the time window\n"
 "  -T endtime    end of the time window\n"
+"  -v            verbose, enable warnings from individual archive fetches\n"
 "  -x            print only stochastic averages for counter metrics\n"
 "  -y            print sample count for each metric\n"
 "  -z            set reporting timezone to local time of metrics source\n"
@@ -85,31 +86,31 @@ static __pmHashCtl	hashlist;
 static __pmHashCtl	errlist;
 
 /* output format flags */
-static unsigned int	stocaveflag = 0;	/* no stochastic counter ave */
-static unsigned int	timeaveflag = 1;	/* use time counter ave */
-static unsigned int	maxflag = 0;		/* no maximum */
-static unsigned int	minflag = 0;		/* no minimum */
-static unsigned int	sumflag = 0;		/* no sum */
-static unsigned int	maxtimeflag = 0;	/* no timestamp for maximum */
-static unsigned int	mintimeflag = 0;	/* no timestamp for minimum */
-static unsigned int	countflag = 0;		/* no count */
-static unsigned int	warnflag = 1;		/* warnings are on by default */
-static unsigned int	delimiter = ' ';	/* output field separator */
-static unsigned int	nbins = 0;		/* number of distribution bins */
-static unsigned int	precision = 3;		/* number of digits after "." */
+static unsigned int	stocaveflag;	/* no stochastic counter ave */
+static unsigned int	timeaveflag = 1;/* use time counter ave */
+static unsigned int	maxflag;	/* no maximum */
+static unsigned int	minflag;	/* no minimum */
+static unsigned int	sumflag;	/* no sum */
+static unsigned int	maxtimeflag;	/* no timestamp for maximum */
+static unsigned int	mintimeflag;	/* no timestamp for minimum */
+static unsigned int	countflag;	/* no count */
+static unsigned int	warnflag;	/* warnings are off by default */
+static unsigned int	delimiter = ' ';/* output field separator */
+static unsigned int	nbins;		/* number of distribution bins */
+static unsigned int	precision = 3;	/* number of digits after "." */
 
 /* time window stuff */
-static int		sflag = 0;
-static int		tflag = 0;
-static int		dayflag = 0;
-static struct timeval	logstart = {0, 0};
-static struct timeval   logend = {0, 0};
-static struct timeval	windowstart = {0, 0};
-static struct timeval   windowend = {0, 0};
+static int		sflag;
+static int		tflag;
+static int		dayflag;
+static struct timeval	logstart;
+static struct timeval   logend;
+static struct timeval	windowstart;
+static struct timeval   windowend;
 static char		timebuf[32];		/* for pmCtime result + .xxx */
 
 /* duration of log */
-static double		logspan = 0.0;
+static double		logspan;
 
 /* time manipulation */
 static int
@@ -191,6 +192,31 @@ printstamp(struct timeval *stamp, int delimiter)
     }
 }
 
+static void
+printheaders(void)
+{
+    printf("metric");
+    if (stocaveflag)
+	printf("%cstochastic_average", delimiter);
+    if (timeaveflag)
+	printf("%ctime_average", delimiter);
+    if (sumflag)
+	printf("%csum", delimiter);
+    if (minflag)
+	printf("%cminimum", delimiter);
+    if (mintimeflag)
+	printf("%cminimum_time", delimiter);
+    if (maxflag)
+	printf("%cmaximum", delimiter);
+    if (maxtimeflag)
+	printf("%cmaximum_time", delimiter);
+    if (countflag)
+	printf("%ccount", delimiter);
+    if (nbins)
+	printf("%cbins", delimiter);
+    printf("%cunits\n", delimiter);
+}
+
 static void
 printsummary(const char *name)
 {
@@ -890,9 +916,11 @@ main(int argc, char *argv[])
     char		*pmnsfile = PM_NS_DEFAULT;
     char		*startstr = NULL;
     char		*endstr = NULL;
+    char		*endnum = NULL;
     char		*msg = NULL;
     int			errflag = 0;
     int			lflag = 0;		/* no label by default */
+    int			Hflag = 0;		/* no header by default */
     pmResult		*result;
     pmLogLabel		label;			/* get hostname for archives */
     struct timeval 	timespan = {0, 0};
@@ -902,21 +930,23 @@ main(int argc, char *argv[])
 
     __pmSetProgname(argv[0]);
 
-    while ((c = getopt(argc, argv, "abB:D:fFiIlmMNn:p:rsS:T:xyzZ:?")) != EOF) {
+    while ((c = getopt(argc, argv, "abB:D:fFHiIlmMNn:p:rsS:T:vxyzZ:?")) != EOF) {
 	switch (c) {
 
 	case 'a':	/* provide all information */
 	    stocaveflag = timeaveflag = lflag = countflag = minflag = maxflag = 1;
+	    sumflag = 0;
 	    break;
 
 	case 'b':	/* use both averages */
 	    stocaveflag = 1;
 	    timeaveflag = 1;
+	    sumflag = 0;
 	    break;
 
 	case 'B':	/* number of distribution bins */
-	    sts = (int)strtol(optarg, &endstr, 10);
-	    if (*endstr != '\0' || sts < 0) {
+	    sts = (int)strtol(optarg, &endnum, 10);
+	    if (*endnum != '\0' || sts < 0) {
 		fprintf(stderr, "%s: -B requires postive numeric argument\n", pmProgname);
 		errflag++;
 	    }
@@ -943,6 +973,10 @@ main(int argc, char *argv[])
 	    delimiter = ',';
 	    break;
 
+	case 'H':	/* print columns headings */
+	    Hflag = 1;
+	    break;
+
 	case 'i':	/* print timestamp for minimum */
 	    mintimeflag = 1;
 	    break;
@@ -972,8 +1006,8 @@ main(int argc, char *argv[])
 	    break;
 
 	case 'p':	/* number of digits after decimal point */
-	    precision = (unsigned int)strtol(optarg, &endstr, 10);
-	    if (*endstr != '\0') {
+	    precision = (unsigned int)strtol(optarg, &endnum, 10);
+	    if (*endnum != '\0') {
 		fprintf(stderr, "%s: -p requires numeric argument\n", pmProgname);
 		errflag++;
 	    }
@@ -994,9 +1028,14 @@ main(int argc, char *argv[])
 	    tflag = 1;
 	    break;
 
+	case 'v':	/* verbose "fetch" warnings reported */
+	    warnflag = 1;
+	    break;
+
 	case 'x':	/* use only stochastic counter averages */
 	    stocaveflag = 1;
 	    timeaveflag = 0;
+	    sumflag = 0;
 	    break;
 
 	case 'y':	/* print sample count */
@@ -1083,8 +1122,6 @@ main(int argc, char *argv[])
 	fprintf(stderr, "         produce complete information.  Continuing and hoping for the best.\n\n");
 	fflush(stderr);
     }
-    else
-	logspan = tosec(logend) - tosec(logstart);
 
     if (tflag || sflag) {
 	if (pmParseTimeWindow(startstr, endstr, NULL, NULL, &logstart, &logend,
@@ -1109,6 +1146,7 @@ main(int argc, char *argv[])
 	windowstart = logstart;
 	windowend = logend;
     }
+    logspan = tosec(windowend) - tosec(windowstart);
 
     if ((sts = pmSetMode(PM_MODE_FORW, &windowstart, 0)) < 0) {
 	fprintf(stderr, "%s: pmSetMode failed: %s\n", pmProgname, pmErrStr(sts));
@@ -1188,6 +1226,9 @@ main(int argc, char *argv[])
 	exitstatus = 1;
     }
 
+    if (Hflag)
+	printheaders();
+
     if (optind >= argc) {	/* print all results */
 	if ((sts = pmTraversePMNS("", printsummary)) < 0) {
 	    fprintf(stderr, "%s: PMNS traversal failed: %s\n", pmProgname, pmErrStr(sts));
