@@ -14,16 +14,11 @@
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
- * 
- * Contact information: Silicon Graphics, Inc., 1500 Crittenden Lane,
- * Mountain View, CA 94043, USA, or: http://www.sgi.com
  */
 
-#ident "$Id: pmnsutil.c,v 1.2 2006/06/15 08:05:47 makc Exp $"
-
 #include "pmapi.h"
 #include "impl.h"
-#include "./pmnsutil.h"
+#include "pmnsutil.h"
 
 static FILE	*outf;
 
