@@ -1,8 +1,8 @@
-/***************************************************************************
- * pmval - performance metrics value dumper
- ***************************************************************************
+/*
+ * pmval - simple performance metrics value dumper
  *
  * Copyright (c) 1995-2001 Silicon Graphics, Inc.  All Rights Reserved.
+ * Copyright (c) 2008-2009 Aconex.  All Rights Reserved.
  * 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -13,39 +13,20 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  */
 
 #include <math.h>
 #include "pmapi.h"
 #include "impl.h"
-
-#if defined(HAVE_PMTIME)
-static void talk_to_pmtime(int *);
-#elif defined(HAVE_KMTIME)
-#include "kmtime.h"
-static void talk_to_kmtime(int *);
-#endif
+#include "pmtime.h"
 
 /***************************************************************************
  * constants
  ***************************************************************************/
 
-#define FALSE	0
-#define TRUE	1
-
-#define	START	-1
-#define STANDBY	0
-#define FORW	1
-#define BACK	2
-#define MOVING	3
-#define ENDLOG	4
-
 #define ALL_SAMPLES	-1
 
+static char *options = "A:a:D:df:gh:i:n:O:p:rs:S:t:T:U:w:zZ:?";
 static char usage[] =
     "Usage: %s [options] metricname\n\n"
     "Options:\n"
@@ -54,17 +35,13 @@ static char usage[] =
     "  -d            delay, pause between updates for archive replay\n"
     "  -f N          fixed precision output format with N digits to the\n"
     "                right of the decimal point\n"
-#if defined(HAVE_PMTIME) || defined(HAVE_KMTIME)
     "  -g            start in GUI mode with new time control\n"
-#endif
     "  -h host       metrics source is PMCD on host\n"
     "  -i instance   metric instance or list of instances - elements in an\n"
     "                instance list are separated by commas or whitespace\n"
     "  -n pmnsfile   use an alternative PMNS\n"
     "  -O offset     initial offset into the time window\n"
-#if defined(HAVE_PMTIME) || defined(HAVE_KMTIME)
-    "  -p port       port name for connection to existing time control\n"
-#endif
+    "  -p port       port number for connection to existing time control\n"
     "  -r            output raw counter values\n"
     "  -S starttime  start of the time window\n"
     "  -s samples    terminate after this many samples\n"
@@ -110,116 +87,25 @@ typedef struct {
  * Globals
  ***************************************************************************/
 
-static char		*archive = NULL;
+static char		*archive;
 static pmLogLabel	label;
 static char		*pmnsfile = PM_NS_DEFAULT;
-static char		*rpt_tz = NULL;
-static char		*rpt_tz_label = NULL;
-static int		pauseFlag = 0;
-static int		raw = 0;
+static char		*rpt_tz;
+static char		*rpt_tz_label;
+static int		pauseFlag;
+static int		raw;
 static int		ahtype = PM_CONTEXT_HOST;	/* archive or host? */
+static int		firstSample = 1;		/* need first sample */
 static int		amode = PM_MODE_INTERP;		/* archive scan mode */
 static char		local[] = "localhost";
-static int		gui = 0;
-static int		rawarchive = 0;
-static int		state = START;
-#if defined(HAVE_PMTIME) || defined(HAVE_KMTIME)
-static char		*control_port = NULL;
-static int		control_fd;
-#if defined(HAVE_PMTIME)
-static pmTime		pmtime;
-static int		lastdeltaunits;	/* from -t or pmtime.vcrmode */
-static int		lastdelta;	/* from -t or pmtime.delta */
-#elif defined(HAVE_KMTIME)
-static int		kmport;
-static kmTime		*kmtime;
-static struct timeval	lastkmdelta;	/* from -t or kmtime.delta */
-#endif
-#endif
+static int		gui;
+static int		rawarchive;
+static int		port = -1;
+static pmTime		*pmtime;
+static pmTimeControls	controls;
 static struct timeval	last = {INT_MAX, 999999};	/* end time for log */
 static int		fixed = -1;
 
-/***************************************************************************
- * timing functions
- ***************************************************************************/
-
-/* add timevals */
-static struct timeval
-tadd(struct timeval t1, struct timeval t2)
-{
-    t1.tv_sec += t2.tv_sec;
-    t1.tv_usec += t2.tv_usec;
-    if (t1.tv_usec > 1000000) {
-	(t1.tv_sec)++;
-	t1.tv_usec -= 1000000;
-    }
-    return t1;
-}
-
-/* subtract timevals */
-static struct timeval
-tsub(struct timeval t1, struct timeval t2)
-{
-    t1.tv_usec -= t2.tv_usec;
-    if (t1.tv_usec < 0) {
-	t1.tv_usec += 1000000;
-	t1.tv_sec--;
-    }
-    t1.tv_sec -= t2.tv_sec;
-    return t1;
-}
-
-/*
- * a : b for struct timevals ... <0 for a<b, ==0 for a==b, >0 for a>b
- */
-static int
-tcmp(struct timeval *a, struct timeval *b)
-{
-    int		res;
-
-    res = (int)(a->tv_sec - b->tv_sec);
-    if (res == 0)
-	res = (int)(a->tv_usec - b->tv_usec);
-    return res;
-}
-
-/* convert timeval to seconds */
-static double
-tosec(struct timeval t)
-{
-    return t.tv_sec + (t.tv_usec / 1000000.0);
-}
-
-/* convert timeval to timespec */
-static struct timespec *
-tospec(struct timeval tv, struct timespec *ts)
-{
-    ts->tv_nsec = tv.tv_usec * 1000;
-    ts->tv_sec = tv.tv_sec;
-    return ts;
-}
-
-
-/* sleep until given timeval */
-static void
-sleeptill(struct timeval sched)
-{
-    int sts;
-    struct timeval curr;	/* current time */
-    struct timespec delay;	/* interval to sleep */
-    struct timespec left;	/* remaining sleep time */
-
-    gettimeofday(&curr, NULL);
-    tospec(tsub(sched, curr), &delay);
-    for (;;) {		/* loop to catch early wakeup by nanosleep */
-	sts = nanosleep(&delay, &left);
-	if (sts == 0 || (sts < 0 && errno != EINTR))
-	    break;
-	delay = left;
-    }
-}
-
-
 /***************************************************************************
  * processing fetched values
  ***************************************************************************/
@@ -235,7 +121,6 @@ compare(const void *pair1, const void *pair2)
     return 0;
 }
 
-
 /* Does the Context have names for all instances in the pmValueSet? */
 static int		/* 1 yes, 0 no */
 chkinsts(Context *x, pmValueSet *vs)
@@ -256,7 +141,6 @@ chkinsts(Context *x, pmValueSet *vs)
     return 1;
 }
 
-
 /***************************************************************************
  * interface to performance metrics API
  ***************************************************************************/
@@ -336,7 +220,6 @@ initinsts(Context *x)
     }
 }
 
-
 /* Initialize API and fill in internal description for given Context. */
 static void
 initapi(Context *x)
@@ -374,48 +257,6 @@ initapi(Context *x)
     }
 }
 
-#if defined(HAVE_PMTIME)	/* let pmtime control know we are done */
-static void
-ack_tctl(struct timeval *now)
-{
-    int		sts;
-
-    if ((sts = pmTimeSendAck(now)) < 0) {
-	if (sts == -EPIPE)
-	    fprintf(stderr, "\n%s: Time Controller has exited, goodbye\n",
-		pmProgname);
-	else
-	    fprintf(stderr, "\n%s: pmTimeSendAck: %s\n", pmProgname, pmErrStr(sts));
-	exit(EXIT_FAILURE);
-    }
-}
-#elif defined(HAVE_KMTIME)	/* let kmtime control know we are done */
-static void
-ack_tctl(struct timeval *now)
-{
-    int		sts;
-
-    /*
-     * DO NOT send back ACKs for timestamps NOT sent from kmtime!  (i.e.
-     * we need to ignore the "now" parameter past in, for kmtime).
-     * pmtime must allow different times? (not checking ACK timestamps?)
-     * The pmFetch seems to modify (at least) the tv_usec component when
-     * interpolating in live mode...
-     */
-    if ((sts = kmTimeSendAck(control_fd, &kmtime->position)) < 0) {
-	if (sts == -EPIPE)
-	    fprintf(stderr, "\n%s: Time Controller has exited, goodbye\n",
-		pmProgname);
-	else
-	    fprintf(stderr, "\n%s: kmTimeSendAck: %s\n", pmProgname, pmErrStr(sts));
-	exit(EXIT_FAILURE);
-    }
-}
-#else
-#define ack_tctl(now)		do { } while (0)
-#endif
-
-
 /* Fetch metric values. */
 static int
 getvals(Context *x,		/* in - full pm description */
@@ -458,11 +299,7 @@ getvals(Context *x,		/* in - full pm description */
 
     if (e < 0) {
 	if (e == PM_ERR_EOL && gui) {
-	    ack_tctl(&last);
-	    if (state != ENDLOG) {
-		printf("\n[Time Control] End of Archive ...\n");
-		state = ENDLOG;
-	    }
+	    pmTimeStateBounds(&controls, pmtime);
 	    return -1;
 	}
 	if (rawarchive)
@@ -473,7 +310,7 @@ getvals(Context *x,		/* in - full pm description */
     }
 
     if (gui)
-	ack_tctl(&r->timestamp);
+	pmTimeStateAck(&controls, pmtime);
 
     if ((double)r->timestamp.tv_sec + (double)r->timestamp.tv_usec/1000000 >
 	(double)last.tv_sec + (double)last.tv_usec/1000000) {
@@ -505,6 +342,12 @@ getvals(Context *x,		/* in - full pm description */
     return i;
 }
 
+static void
+timestep(struct timeval delta)
+{
+    firstSample = 1;
+}
+
 
 /***************************************************************************
  * output
@@ -529,63 +372,6 @@ howide(int type)
     }
 }
 
-/*
- * Get Extended Time Base interval and Units from a timeval
- */
-#define SECS_IN_24_DAYS 2073600.0
-
-static void
-getXTBintervalFromTimeval(int *ival, int *mode, struct timeval *tval)
-{
-    double tmp_ival = tval->tv_sec + tval->tv_usec / 1000000.0;
-
-    if (tmp_ival > SECS_IN_24_DAYS) {
-	*ival = (int)tmp_ival;
-	*mode = (*mode & 0x0000ffff) | PM_XTB_SET(PM_TIME_SEC);
-    }
-    else {
-	*ival = (int)(tmp_ival * 1000.0);
-	*mode = (*mode & 0x0000ffff) | PM_XTB_SET(PM_TIME_MSEC);
-    }
-}
-
-#ifdef HAVE_PMTIME
-/*
- * Get the interval in seconds
- */
-static double
-getXTBinSeconds(int *ival, int *mode)
-{
-    double rval = 0.0;
-
-    switch(PM_XTB_GET(*mode)) {
-    case PM_TIME_NSEC:
-	rval = *ival / 1000000000.0;
-	break;
-    case PM_TIME_USEC:
-	rval = *ival / 1000000.0;
-	break;
-    case PM_TIME_SEC:
-	rval = (double)*ival;
-	break;
-    case PM_TIME_MIN:
-	rval = *ival * 60;
-	break;
-    case PM_TIME_HOUR:
-	rval = *ival * 3600;
-	break;
-    case PM_TIME_MSEC:
-    default:
-	/* default (not XTB) is milliseconds */
-	rval = *ival / 1000.0;
-	break;
-    }
-    
-    return rval;
-}
-#endif
-
-
 /* Print parameter values as output header. */
 static void
 printhdr(Context *x, long smpls, struct timeval delta, struct timeval first)
@@ -650,14 +436,9 @@ printhdr(Context *x, long smpls, struct timeval delta, struct timeval first)
     /* sample count */
     if (smpls == ALL_SAMPLES) printf("samples:   all\n");
     else printf("samples:   %ld\n", smpls);
-    if (smpls != ALL_SAMPLES && smpls > 1 && (ahtype != PM_CONTEXT_ARCHIVE || amode == PM_MODE_INTERP)) {
-	printf("interval:  %1.2f sec\n", tosec(delta));
-#if defined(HAVE_PMTIME)
-	getXTBintervalFromTimeval(&lastdelta, &lastdeltaunits, &delta);
-#elif defined(HAVE_KMTIME)
-	lastkmdelta = delta;
-#endif
-    }
+    if (smpls != ALL_SAMPLES && smpls > 1 &&
+	(ahtype != PM_CONTEXT_ARCHIVE || amode == PM_MODE_INTERP))
+	printf("interval:  %1.2f sec\n", __pmtimevalToReal(&delta));
 }
 
 /* Print instance identifier names as column labels. */
@@ -886,7 +667,8 @@ printrates(Context *x,
     double  delta;
 
     /* compute delta from timestamps and convert units */
-    delta = x->scale * (tosec(stamp1) - tosec(stamp2));
+    delta = x->scale *
+	    (__pmtimevalToReal(&stamp1) - __pmtimevalToReal(&stamp2));
 
     /* null instance domain */
     if (x->desc.indom == PM_INDOM_NULL) {
@@ -1021,9 +803,6 @@ getargs(int		argc,		/* in - command line argument count */
     struct timeval  first;
     pmMetricSpec   *msp;
     char	    *msg;
-#ifdef HAVE_PMTIME
-    struct stat	statbuf;
-#endif
 
     /* fill in default values */
     cntxt->iall = 1;
@@ -1035,7 +814,7 @@ getargs(int		argc,		/* in - command line argument count */
     *cols = 0;
 
     /* extract command-line arguments */
-    while ((c = getopt(argc, argv, "A:a:D:df:gh:i:n:O:p:rs:S:t:T:U:w:zZ:?")) != EOF) {
+    while ((c = getopt(argc, argv, options)) != EOF) {
 	switch (c) {
 
 	case 'A':		/* sample alignment */
@@ -1075,11 +854,9 @@ getargs(int		argc,		/* in - command line argument count */
 	   fixed = d;
 	   break;
 
-#if defined(HAVE_PMTIME) || defined(HAVE_KMTIME)
 	case 'g':
 	    gui = 1;
 	    break;
-#endif
 
 	case 'h':		/* host name */
 	    if (++src > 1) {
@@ -1115,31 +892,12 @@ getargs(int		argc,		/* in - command line argument count */
 	    break;
 
 	case 'p':		/* port for slave of existing time control */
-#if defined(HAVE_PMTIME)
-	    if ((sts = stat(optarg, &statbuf)) < 0) {
-		fprintf(stderr, "%s: Error: can not access time control port \"%s\": %s\n",
-		    pmProgname, optarg, pmErrStr(-errno));
-		errflag++;
-	    }
-	    else if ((statbuf.st_mode & S_IFSOCK) != S_IFSOCK) {
-		fprintf(stderr, "%s: Error: time control port \"%s\" is not a socket\n",
-		    pmProgname, optarg);
-		errflag++;
-	    }
-	    else
-		control_port = optarg;
-#elif defined(HAVE_KMTIME)
-	    kmport = (int)strtol(optarg, &endnum, 10);
-	    if (*endnum != '\0' || kmport < 0) {
-		fprintf(stderr, "%s: Error: invalid kmtime port \"%s\": %s\n",
+	    port = (int)strtol(optarg, &endnum, 10);
+	    if (*endnum != '\0' || port < 0) {
+		fprintf(stderr, "%s: Error: invalid pmtime port \"%s\": %s\n",
 			pmProgname, optarg, pmErrStr(-errno));
 		errflag++;
-	    } else
-	    	control_port = optarg;
-#else
-	    fprintf(stderr, "%s: Sorry, no time control support\n", pmProgname);
-	    errflag++;
-#endif
+	    }
 	    break;
 
 	case 'r':		/* raw */
@@ -1267,9 +1025,8 @@ getargs(int		argc,		/* in - command line argument count */
 	    errflag++;
 	}
     }
-#if defined(HAVE_PMTIME) || defined(HAVE_KMTIME)
     else {
-	if (gui == 1 && control_port != NULL) {
+	if (gui == 1 && port != -1) {
 	    fprintf(stderr, "%s: -g cannot be used with -p\n", pmProgname);
 	    errflag++;
 	}
@@ -1278,7 +1035,6 @@ getargs(int		argc,		/* in - command line argument count */
 	    errflag++;
 	}
     }
-#endif
 
     if (errflag) {
 	fprintf(stderr, usage, pmProgname);
@@ -1341,7 +1097,6 @@ getargs(int		argc,		/* in - command line argument count */
 	    printf("Note: timezone set to local timezone of host \"%s\"\n\n", host);
 	    rpt_tz_label = host;
 	}
-	pmWhichZone(&rpt_tz);
     }
     else if (tz != NULL) {
 	if ((tzh = pmNewZone(tz)) < 0) {
@@ -1350,20 +1105,27 @@ getargs(int		argc,		/* in - command line argument count */
 	    exit(EXIT_FAILURE);
 	}
 	printf("Note: timezone set to \"TZ=%s\"\n\n", tz);
-	pmWhichZone(&rpt_tz);
     }
     else printf("\n");
 
+    pmWhichZone(&rpt_tz);
+    if (!rpt_tz_label)
+	rpt_tz_label = local;
+
     if (pmParseTimeWindow(Sflag, Tflag, Aflag, Oflag,
 			   &logStart, &last,
 			   &first, &last, posn, &msg) < 0) {
-	fprintf(stderr, msg);
+	fprintf(stderr, "%s", msg);
 	exit(EXIT_FAILURE);
     }
 
-    if (!(gui || control_port != NULL) &&
-	*smpls == ALL_SAMPLES && last.tv_sec != INT_MAX && amode != PM_MODE_FORW) {
-	*smpls = (long)((tosec(last) - tosec(*posn)) / tosec(*delta));
+    if (!(gui || port != -1) &&
+	*smpls == ALL_SAMPLES &&
+	last.tv_sec != INT_MAX &&
+	amode != PM_MODE_FORW) {
+
+	*smpls = (long)((__pmtimevalToReal(&last) - __pmtimevalToReal(posn)) /
+		__pmtimevalToReal(delta));
 	/* if end is before start, no samples thanks */
 	if (*smpls < 0) *smpls = 0;
 	/* counters require 2 samples to produce reported sample */
@@ -1372,121 +1134,20 @@ getargs(int		argc,		/* in - command line argument count */
 #ifdef PCP_DEBUG
 	if (pmDebug & DBG_TRACE_APPL0)
 	    fprintf(stderr, "getargs: first=%.6f posn=%.6f last=%.6f\ngetargs: delta=%.6f samples=%ld\n",
-	    tosec(first), tosec(*posn), tosec(last), tosec(*delta), *smpls);
+	    __pmtimevalToReal(&first), __pmtimevalToReal(posn),
+	    __pmtimevalToReal(&last), __pmtimevalToReal(delta), *smpls);
 #endif
     }
 
-#ifdef HAVE_PMTIME
-    if (gui || control_port != NULL) {
+    if (gui || port != -1) {
 	/* set up pmtime control */
-	int		mode;
-
-	if (msp->isarch == 1)
-	    mode = PM_TCTL_MODE_ARCHIVE;
-	else
-	    mode = PM_TCTL_MODE_HOST;
-
-	pmtime.showdialog = 0;	/* don't expose dialog yet */
-
-	if (gui) {
-	    mode |= PM_TCTL_MODE_NEWMASTER;
-	    control_port = mktemp("/usr/tmp/vcr.XXXXXX");
-	}
-	else
-	    mode |= PM_TCTL_MODE_MASTER;
-
-	getXTBintervalFromTimeval(&pmtime.delta, &pmtime.vcrmode, delta);
-
-	if (msp->isarch == 1) {
-	    pmtime.position = *posn;
-	    pmtime.start = first;
-	    pmtime.finish = last;
-	}
-	else
-	    gettimeofday(&pmtime.position, NULL);
-	if (rpt_tz == NULL) {
-            rpt_tz = __pmTimezone();
-	    if (msp->isarch == 1) {
-		if ((sts = pmNewZone(rpt_tz)) < 0) {
-		    fprintf(stderr, "%s: Cannot set timezone to \"%s\": %s\n",
-			pmProgname, rpt_tz, pmErrStr(sts));
-		    exit(EXIT_FAILURE);
-		}
-	    }
-	}
-	strncpy(pmtime.tz, rpt_tz, sizeof(pmtime.tz));
-	if (rpt_tz_label == NULL)
-	    rpt_tz_label = "localhost";
-	strncpy(pmtime.tzlabel, rpt_tz_label, sizeof(pmtime.tzlabel));
-	if ((control_fd = pmTimeConnect(mode, control_port, &pmtime)) < 0) {
-	    fprintf(stderr, "%s: pmTimeConnect: %s\n", pmProgname, pmErrStr(control_fd));
-	    exit(EXIT_FAILURE);
-	}
-	gui = 1;		/* means using pmtime control from here on */
+	pmtime = pmTimeStateSetup(&controls, ahtype, port, *delta, *posn,
+				    first, last, rpt_tz, rpt_tz_label);
+	controls.stepped = timestep;
+	gui = 1;	/* we're using pmtime control from here on */
     }
-    else 
-#elif defined(HAVE_KMTIME)
-    if (gui || control_port != NULL) {
-	/* set up kmtime control */
-	if (gui)
-	    kmport = -1;
-	kmtime = malloc(sizeof(kmTime));
-	kmtime->magic = KMTIME_MAGIC;
-	kmtime->length = sizeof(kmTime);
-	kmtime->command = KM_TCTL_SET;
-	kmtime->delta = *delta;
-	if (msp->isarch == 1) {
-	    kmtime->source = KM_SOURCE_ARCHIVE;
-	    kmtime->position = *posn;
-	    kmtime->start = first;
-	    kmtime->end = last;
-	} else {
-	    kmtime->source = KM_SOURCE_HOST;
-	    gettimeofday(&kmtime->position, NULL);
-	}
-	if (rpt_tz == NULL) {
-	    rpt_tz = __pmTimezone();
-	    if (msp->isarch == 1) {
-		if ((sts = pmNewZone(rpt_tz)) < 0) {
-		    fprintf(stderr, "%s: Cannot set timezone to \"%s\": %s\n",
-			pmProgname, rpt_tz, pmErrStr(sts));
-		    exit(EXIT_FAILURE);
-		}
-	    }
-	}
-	tzh = strlen(rpt_tz) + 1;
-	if (rpt_tz_label == NULL)
-	    rpt_tz_label = "localhost";
-	kmtime->length += tzh + strlen(rpt_tz_label) + 1;
-	kmtime = realloc(kmtime, kmtime->length);
-	if (!kmtime) {
-	    fprintf(stderr, "%s: realloc: %s\n", pmProgname, strerror(errno));
-	    exit(EXIT_FAILURE);
-	}
-	strcpy(kmtime->data, rpt_tz);
-	strcpy(kmtime->data + tzh, rpt_tz_label);
-	if ((control_fd = kmTimeConnect(kmport, kmtime)) < 0) {
-	    fprintf(stderr, "%s: kmTimeConnect: %s\n",
-		    pmProgname, pmErrStr(control_fd));
-	    exit(EXIT_FAILURE);
-	}
-	kmtime->length = sizeof(kmTime); /* reduce size to header only */
-	kmtime = realloc(kmtime, kmtime->length);
-	gui = 1;		/* means using kmtime control from here on */
-    }
-    else
-#endif
-	if (msp->isarch == 1) {
-	    /* archive, and no time control, go it alone */
-	    int tmp_ival;
-	    int tmp_mode;
-	    getXTBintervalFromTimeval(&tmp_ival, &tmp_mode, delta);
-	    tmp_mode = (tmp_mode & 0xffff0000) | (amode & __PM_MODE_MASK);
-	    if ((sts = pmSetMode(tmp_mode, posn, tmp_ival)) < 0) {
-		fprintf(stderr, "%s: pmSetMode: %s\n", pmProgname, pmErrStr(sts));
-		exit(EXIT_FAILURE);
-	    }
-	}
+    else if (ahtype == PM_CONTEXT_ARCHIVE) /* no time control, go it alone */
+	pmTimeStateMode(amode, *delta, posn);
 }
 
 /***************************************************************************
@@ -1496,19 +1157,15 @@ int
 main(int argc, char *argv[])
 {
     struct timeval  delta;		/* sample interval */
-    struct timespec delay;		/* nanosleep interval */
-    struct timespec left;		/* nanosleep remainder */
     long	    smpls;		/* number of samples */
     int             cols;		/* width of output column */
     struct timeval  now;		/* current task start time */
-    struct timeval  sched;		/* next task scheduled time */
     Context	    cntxt;		/* performance metric description */
     pmResult	    *rslt1;		/* current values */
     pmResult	    *rslt2;		/* previous values */
-    int		    first = 1;		/* need first sample */
     int		    forever;
     int		    idx1;
-    int		    idx2;
+    int		    idx2 = 0;		/* initialize to pander to gcc */
     int		    no_values = 0;
 
     __pmSetProgname(argv[0]);
@@ -1519,40 +1176,28 @@ main(int argc, char *argv[])
     initapi(&cntxt);
     initinsts(&cntxt);
 
-    /* seems safe enough at this point to expose the Time Control dialog... */
-#if defined(HAVE_PMTIME)
-    if (gui)
-	pmTimeShowDialog(1);
-#elif defined(HAVE_KMTIME)
-    if (gui)
-	kmTimeShowDialog(control_fd, 1);
-#endif
-
     if (cols <= 0) cols = howide(cntxt.desc.type);
 
-    if ((fixed == 0 && fixed > cols) ||
-        (fixed > 0 && fixed > cols - 2)) {
-	fprintf(stderr, "%s: -f %d too large for column width %d\n", pmProgname, fixed, cols);
+    if ((fixed == 0 && fixed > cols) || (fixed > 0 && fixed > cols - 2)) {
+	fprintf(stderr, "%s: -f %d too large for column width %d\n",
+		pmProgname, fixed, cols);
 	exit(EXIT_FAILURE);
     }
 
     printhdr(&cntxt, smpls, delta, now);
 
     /* wait till time for first sample */
-    if (archive == NULL )
-	sleeptill(now);
+    if (archive == NULL)
+	__pmtimevalPause(now);
 
     /* main loop fetching and printing sample values */
     while (forever || (smpls-- > 0)) {
-#if defined(HAVE_PMTIME)
-	talk_to_pmtime(&first);
-#elif defined(HAVE_KMTIME)
-	talk_to_kmtime(&first);
-#endif
-	if (first) {
+	if (gui)
+	    pmTimeStateVector(&controls, pmtime);
+	if (firstSample) {
 	    if ((idx2 = getvals(&cntxt, &rslt2)) >= 0) {
 		/* first-time success */
-		first = 0;
+		firstSample = 0;
 		if (cntxt.desc.indom != PM_INDOM_NULL)
 		    printlabels(&cntxt, cols);
 		if (raw || (cntxt.desc.sem != PM_SEM_COUNTER)) {
@@ -1581,25 +1226,18 @@ main(int argc, char *argv[])
 	}
 
 	/* wait till time for sample */
-	if (pauseFlag) {
-	    nanosleep(tospec(delta, &delay), &left);
-	}
-	else if (archive == NULL) {
-	    sched = tadd(now,delta);
-	    now = sched;
-	    sleeptill(sched);
-	}
+	if (!gui && (pauseFlag || archive == NULL))
+	    __pmtimevalSleep(delta);
 
-	if (first)
-	    /* keep trying */
-	    continue;
+	if (firstSample)
+	    continue;	/* keep trying */
 
 	/* next sample */
 	if ((idx1 = getvals(&cntxt, &rslt1)) == -2)
 		/* out the end of the window */
 		break;
 	else if (idx1 < 0) {
-	    first = 1;
+	    firstSample = 1;
 	    continue;
 	}
 
@@ -1628,223 +1266,5 @@ main(int argc, char *argv[])
 	idx2 = idx1;
     }
 
-    exit(first == 0);
+    exit(firstSample == 0);
 }
-
-#ifdef HAVE_PMTIME
-static void talk_to_pmtime(int *first)
-{
-    if (gui) {
-	for ( ; ; ) {
-	    int sts;
-	    int fetch = 0;
-	    int cmd = pmTimeRecv(&pmtime);
-	    if (cmd < 0) {
-		fprintf(stderr, "\n%s: Time Control dialog has terminated: %s\n",
-			pmProgname, pmErrStr(cmd));
-		fprintf(stderr, "Sorry.\n");
-            	exit(EXIT_FAILURE);
-	    }
-
-	    switch (cmd) {
-	    case PM_TCTL_SET:
-		if (state == ENDLOG)
-		    state = STANDBY;
-		else if (state == FORW)
-		    state = START;
-		break;
-
-	    case PM_TCTL_STEP:
-		if (pmtime.delta < 0) {
-		    if (state != BACK) {
-			printf("\n[Time Control] Rewind/Reverse ...\n");
-			state = BACK;
-		    }
-		}
-		else if (state != FORW && state != ENDLOG) {
-		    if (ahtype == PM_CONTEXT_ARCHIVE) {
-			if (state != STANDBY)
-			    printf("\n[Time Control] Repositioned in archive ...\n");
-		    }
-		    else {
-			printf("\n[Time Control] Resume ...\n");
-		    }
-		    if (pmtime.delta != lastdelta ||
-		        PM_XTB_GET(pmtime.vcrmode) != PM_XTB_GET(lastdeltaunits))
-			printf("new interval:  %1.2f sec\n",
-				getXTBinSeconds(&pmtime.delta, &pmtime.vcrmode));
-
-		    if (ahtype == PM_CONTEXT_ARCHIVE) {
-			int setmode = PM_MODE_INTERP | (pmtime.vcrmode & 0xffff0000);
-			if ((sts = pmSetMode(setmode, &pmtime.position, pmtime.delta)) < 0) {
-			    fprintf(stderr, "%s: pmSetMode: %s\n", pmProgname, pmErrStr(sts));
-			    exit(EXIT_FAILURE);
-			}
-		    }
-		    lastdeltaunits = pmtime.vcrmode & 0xffff0000;
-		    lastdelta = pmtime.delta;
-		    state = FORW;
-		    *first = 1;
-		}
-
-		if (state == BACK || state == ENDLOG) {
-		    /*
-		     * for EOL and reverse travel, no pmFetch,
-		     * so ack here
-		     */
-		    ack_tctl(&pmtime.position);
-		    break;
-		}
-		fetch = 1;
-		break;
-
-	    case PM_TCTL_TZ:
-		if ((sts = pmNewZone(pmtime.tz)) < 0) {
-		    fprintf(stderr, "%s: Warning: cannot set timezone to \"%s\": %s\n",
-			pmProgname, pmtime.tz, pmErrStr(sts));
-		}
-		break;
-
-	    case PM_TCTL_VCRMODE:
-		/* something has changed ... suppress reporting */
-		if ((pmtime.vcrmode & __PM_MODE_MASK) == PM_TCTL_VCRMODE_DRAG)
-		    state = MOVING;
-		else if (state != MOVING)
-		    state = STANDBY;
-		break;
-
-	    /*
-	     * safely and silently ignore these
-	     */
-	    case PM_TCTL_SHOWDIALOG:
-		break;
-
-	    case PM_TCTL_SKIP:
-	    case PM_TCTL_BOUNDS:
-	    case PM_TCTL_ACK:
-		break;
-
-	    default:
-		printf("pmTimeRecv: cmd %d?\n", cmd);
-		break;
-	    }
-	    if (fetch)
-		break;
-	}
-    }
-}
-#endif
-
-#ifdef HAVE_KMTIME
-static void talk_to_kmtime(int *first)
-{
-    if (gui) {
-	for ( ; ; ) {
-	    int sts;
-	    int fetch = 0;
-	    int cmd = kmTimeRecv(control_fd, &kmtime);
-
-	    if (cmd < 0) {
-		fprintf(stderr, "\n%s: Time Control dialog exited, sorry.\n",
-			pmProgname);
-            	exit(EXIT_FAILURE);
-	    }
-
-	    switch (kmtime->command) {
-	    case KM_TCTL_SET:
-		if (state == ENDLOG)
-		    state = STANDBY;
-		else if (state == FORW)
-		    state = START;
-		break;
-
-	    case KM_TCTL_STEP:
-		if (kmtime->state == KM_STATE_BACKWARD) {
-		    if (state != BACK) {
-			printf("\n[Time Control] Rewind/Reverse ...\n");
-			state = BACK;
-		    }
-		}
-		else if (state != FORW && state != ENDLOG) {
-		    if (ahtype == PM_CONTEXT_ARCHIVE) {
-			if (state != STANDBY)
-			    printf("\n[Time Control] Repositioned in archive ...\n");
-		    } else {
-			printf("\n[Time Control] Resume ...\n");
-		    }
-		    if (tcmp(&kmtime->delta, &lastkmdelta) != 0)
-			printf("new interval:  %1.2f sec\n", tosec(kmtime->delta));
-
-		    if (ahtype == PM_CONTEXT_ARCHIVE) {
-			int setmode = PM_MODE_INTERP;
-			int delta = kmtime->delta.tv_sec;
-
-			if (kmtime->delta.tv_usec == 0) {
-			    setmode |= PM_XTB_SET(PM_TIME_SEC);
-			} else {
-			    delta = delta * 1000 + kmtime->delta.tv_usec / 1000;
-			    setmode |= PM_XTB_SET(PM_TIME_MSEC);
-			}
-			sts = pmSetMode(setmode, &kmtime->position, delta);
-			if (sts < 0) {
-			    fprintf(stderr, "%s: pmSetMode: %s\n",
-					pmProgname, pmErrStr(sts));
-			    exit(EXIT_FAILURE);
-			}
-		    }
-		    lastkmdelta = kmtime->delta;
-		    state = FORW;
-		    *first = 1;
-		}
-
-		if (state == BACK || state == ENDLOG) {
-		    /*
-		     * for EOL and reverse travel, no pmFetch,
-		     * so ack here
-		     */
-		    ack_tctl(&kmtime->position);
-		    break;
-		}
-		fetch = 1;
-		break;
-
-	    case KM_TCTL_TZ:
-		if ((sts = pmNewZone(kmtime->data)) < 0) {
-		    fprintf(stderr,
-			"%s: Warning: cannot set timezone to \"%s\": %s\n",
-			pmProgname, kmtime->data, pmErrStr(sts));
-		} else {
-		    printf("new timezone: %s (%s)\n", kmtime->data,
-			    kmtime->data + strlen(kmtime->data) + 1);
-		}
-		break;
-
-	    case KM_TCTL_VCRMODE:
-	    case KM_TCTL_VCRMODE_DRAG:
-		/* something has changed ... suppress reporting */
-		if (kmtime->state == KM_TCTL_VCRMODE_DRAG)
-		    state = MOVING;
-		else if (state != MOVING)
-		    state = STANDBY;
-		break;
-
-	    /*
-	     * safely and silently ignore these
-	     */
-	    case KM_TCTL_GUISTYLE:
-	    case KM_TCTL_GUISHOW:
-	    case KM_TCTL_GUIHIDE:
-	    case KM_TCTL_BOUNDS:
-	    case KM_TCTL_ACK:
-		break;
-
-	    default:
-		printf("kmTimeRecv: cmd %x?\n", cmd);
-		break;
-	    }
-	    if (fetch)
-		break;
-	}
-    }
-}
-#endif
